@@ -91,7 +91,7 @@ unsigned Bind::location(NameAST *name, unsigned defaultLocation) const
         return defaultLocation;
 
     else if (DestructorNameAST *dtor = name->asDestructorName())
-        return dtor->identifier_token;
+        return location(dtor->unqualified_name, defaultLocation);
 
     else if (TemplateIdAST *templId = name->asTemplateId())
         return templId->identifier_token;
@@ -337,8 +337,14 @@ FullySpecifiedType Bind::declarator(DeclaratorAST *ast, const FullySpecifiedType
         return type;
 
     std::swap(_declaratorId, declaratorId);
+    bool isAuto = false;
+    if (translationUnit()->cxx0xEnabled())
+        isAuto = type.isAuto();
+
     for (SpecifierListAST *it = ast->attribute_list; it; it = it->next) {
         type = this->specifier(it->value, type);
+        if (type.isAuto())
+            isAuto = true;
     }
     for (PtrOperatorListAST *it = ast->ptr_operator_list; it; it = it->next) {
         type = this->ptrOperator(it->value, type);
@@ -349,9 +355,18 @@ FullySpecifiedType Bind::declarator(DeclaratorAST *ast, const FullySpecifiedType
     type = this->coreDeclarator(ast->core_declarator, type);
     for (SpecifierListAST *it = ast->post_attribute_list; it; it = it->next) {
         type = this->specifier(it->value, type);
+        if (type.isAuto())
+            isAuto = true;
     }
-    // unsigned equals_token = ast->equals_token;
-    ExpressionTy initializer = this->expression(ast->initializer);
+    if (!type->isFunctionType()) {
+        ExpressionTy initializer = this->expression(ast->initializer);
+        if (translationUnit()->cxx0xEnabled() && isAuto) {
+
+            type = initializer;
+            type.setAuto(true);
+        }
+    }
+
     std::swap(_declaratorId, declaratorId);
     return type;
 }
@@ -459,7 +474,7 @@ void Bind::enumerator(EnumeratorAST *ast, Enum *symbol)
     }
 }
 
-bool Bind::visit(ExceptionSpecificationAST *ast)
+bool Bind::visit(DynamicExceptionSpecificationAST *ast)
 {
     (void) ast;
     assert(!"unreachable");
@@ -473,11 +488,15 @@ FullySpecifiedType Bind::exceptionSpecification(ExceptionSpecificationAST *ast,
     if (! ast)
         return type;
 
-    // unsigned throw_token = ast->throw_token;
-    // unsigned lparen_token = ast->lparen_token;
-    // unsigned dot_dot_dot_token = ast->dot_dot_dot_token;
-    for (ExpressionListAST *it = ast->type_id_list; it; it = it->next) {
-        ExpressionTy value = this->expression(it->value);
+    if (DynamicExceptionSpecificationAST *dyn = ast->asDynamicExceptionSpecification()) {
+        // unsigned throw_token = ast->throw_token;
+        // unsigned lparen_token = ast->lparen_token;
+        // unsigned dot_dot_dot_token = ast->dot_dot_dot_token;
+        for (ExpressionListAST *it = dyn->type_id_list; it; it = it->next) {
+            /*ExpressionTy value =*/ this->expression(it->value);
+        }
+    } else if (NoExceptSpecificationAST *no = ast->asNoExceptSpecification()) {
+        /*ExpressionTy value =*/ this->expression(no->expression);
     }
     // unsigned rparen_token = ast->rparen_token;
     return type;
@@ -1080,7 +1099,12 @@ void Bind::lambdaDeclarator(LambdaDeclaratorAST *ast)
         return;
 
 
-    Function *fun = 0; // ### implement me
+    Function *fun = control()->newFunction(0, 0);
+    fun->setStartOffset(tokenAt(ast->firstToken()).begin());
+    fun->setEndOffset(tokenAt(ast->lastToken() - 1).end());
+    if (ast->trailing_return_type)
+        _type = this->trailingReturnType(ast->trailing_return_type, _type);
+    fun->setReturnType(_type);
 
     // unsigned lparen_token = ast->lparen_token;
     FullySpecifiedType type;
@@ -1091,7 +1115,6 @@ void Bind::lambdaDeclarator(LambdaDeclaratorAST *ast)
     }
     // unsigned mutable_token = ast->mutable_token;
     type = this->exceptionSpecification(ast->exception_specification, type);
-    type = this->trailingReturnType(ast->trailing_return_type, type);
 }
 
 bool Bind::visit(TrailingReturnTypeAST *ast)
@@ -1231,11 +1254,79 @@ bool Bind::visit(ForeachStatementAST *ast)
     }
     DeclaratorIdAST *declaratorId = 0;
     type = this->declarator(ast->declarator, type, &declaratorId);
+    const StringLiteral *initializer = 0;
+    if (type.isAuto() && translationUnit()->cxx0xEnabled()) {
+        ExpressionTy exprType = this->expression(ast->expression);
+
+        ArrayType* arrayType = 0;
+        arrayType = exprType->asArrayType();
+
+        if (arrayType != 0)
+            type = arrayType->elementType();
+        else if (ast->expression != 0) {
+            unsigned startOfExpression = ast->expression->firstToken();
+            unsigned endOfExpression = ast->expression->lastToken();
+            const StringLiteral *sl = asStringLiteral(startOfExpression, endOfExpression);
+            const std::string buff = std::string("*") + sl->chars() + ".begin()";
+            initializer = control()->stringLiteral(buff.c_str(), buff.size());
+        }
+    }
 
     if (declaratorId && declaratorId->name) {
         unsigned sourceLocation = location(declaratorId->name, ast->firstToken());
         Declaration *decl = control()->newDeclaration(sourceLocation, declaratorId->name->name);
         decl->setType(type);
+        decl->setInitializer(initializer);
+        block->addMember(decl);
+    }
+
+    /*ExpressionTy initializer =*/ this->expression(ast->initializer);
+    /*ExpressionTy expression =*/ this->expression(ast->expression);
+    this->statement(ast->statement);
+    (void) switchScope(previousScope);
+    return false;
+}
+
+bool Bind::visit(RangeBasedForStatementAST *ast)
+{
+    Block *block = control()->newBlock(ast->firstToken());
+    const unsigned startScopeToken = ast->lparen_token ? ast->lparen_token : ast->firstToken();
+    block->setStartOffset(tokenAt(startScopeToken).end());
+    block->setEndOffset(tokenAt(ast->lastToken()).begin());
+    _scope->addMember(block);
+    ast->symbol = block;
+
+    Scope *previousScope = switchScope(block);
+
+    FullySpecifiedType type;
+    for (SpecifierListAST *it = ast->type_specifier_list; it; it = it->next) {
+        type = this->specifier(it->value, type);
+    }
+    DeclaratorIdAST *declaratorId = 0;
+    type = this->declarator(ast->declarator, type, &declaratorId);
+    const StringLiteral *initializer = 0;
+    if (type.isAuto() && translationUnit()->cxx0xEnabled()) {
+        ExpressionTy exprType = this->expression(ast->expression);
+
+        ArrayType* arrayType = 0;
+        arrayType = exprType->asArrayType();
+
+        if (arrayType != 0)
+            type = arrayType->elementType();
+        else if (ast->expression != 0) {
+            unsigned startOfExpression = ast->expression->firstToken();
+            unsigned endOfExpression = ast->expression->lastToken();
+            const StringLiteral *sl = asStringLiteral(startOfExpression, endOfExpression);
+            const std::string buff = std::string("*") + sl->chars() + ".begin()";
+            initializer = control()->stringLiteral(buff.c_str(), buff.size());
+        }
+    }
+
+    if (declaratorId && declaratorId->name) {
+        unsigned sourceLocation = location(declaratorId->name, ast->firstToken());
+        Declaration *decl = control()->newDeclaration(sourceLocation, declaratorId->name->name);
+        decl->setType(type);
+        decl->setInitializer(initializer);
         block->addMember(decl);
     }
 
@@ -1594,6 +1685,13 @@ bool Bind::visit(SizeofExpressionAST *ast)
     return false;
 }
 
+bool Bind::visit(PointerLiteralAST *ast)
+{
+    (void) ast;
+    // unsigned literal_token = ast->literal_token;
+    return false;
+}
+
 bool Bind::visit(NumericLiteralAST *ast)
 {
     (void) ast;
@@ -1753,6 +1851,8 @@ bool Bind::visit(SimpleDeclarationAST *ast)
                 name = elabTypeSpec->name->name;
             }
 
+            ensureValidClassName(&name, sourceLocation);
+
             ForwardClassDeclaration *decl = control()->newForwardClassDeclaration(sourceLocation, name);
             setDeclSpecifiers(decl, type);
             _scope->addMember(decl);
@@ -1784,6 +1884,16 @@ bool Bind::visit(SimpleDeclarationAST *ast)
             if (declaratorId && declaratorId->name)
                 fun->setName(declaratorId->name->name); // update the function name
         }
+        else if (declTy.isAuto()) {
+            const ExpressionAST *initializer = it->value->initializer;
+            if (!initializer)
+                translationUnit()->error(location(declaratorId->name, ast->firstToken()), "auto-initialized variable must have an initializer");
+            else {
+                unsigned startOfExpression = initializer->firstToken();
+                unsigned endOfExpression = initializer->lastToken();
+                decl->setInitializer(asStringLiteral(startOfExpression, endOfExpression));
+            }
+        }
 
         if (_scope->isClass()) {
             decl->setVisibility(_visibility);
@@ -1988,7 +2098,16 @@ bool Bind::visit(ExceptionDeclarationAST *ast)
     }
     DeclaratorIdAST *declaratorId = 0;
     type = this->declarator(ast->declarator, type, &declaratorId);
+
+    const Name *argName = 0;
+    if (declaratorId && declaratorId->name)
+        argName = declaratorId->name->name;
+    Argument *arg = control()->newArgument(location(declaratorId, ast->firstToken()), argName);
+    arg->setType(type);
+    _scope->addMember(arg);
+
     // unsigned dot_dot_dot_token = ast->dot_dot_dot_token;
+
     return false;
 }
 
@@ -2074,6 +2193,7 @@ bool Bind::visit(NamespaceAST *ast)
     Namespace *ns = control()->newNamespace(sourceLocation, namespaceName);
     ns->setStartOffset(tokenAt(sourceLocation).end()); // the scope starts after the namespace or the identifier token.
     ns->setEndOffset(tokenAt(ast->lastToken() - 1).end());
+    ns->setInline(ast->inline_token != 0);
     ast->symbol = ns;
     _scope->addMember(ns);
 
@@ -2520,8 +2640,7 @@ bool Bind::visit(SimpleNameAST *ast)
 
 bool Bind::visit(DestructorNameAST *ast)
 {
-    const Identifier *id = identifier(ast->identifier_token);
-    _name = control()->destructorNameId(id);
+    _name = control()->destructorNameId(name(ast->unqualified_name));
     ast->name = _name;
     return false;
 }
@@ -2550,6 +2669,20 @@ bool Bind::visit(TemplateIdAST *ast)
 bool Bind::visit(SimpleSpecifierAST *ast)
 {
     switch (tokenKind(ast->specifier_token)) {
+        case T_IDENTIFIER: {
+                const Identifier *id = tokenAt(ast->specifier_token).identifier;
+                if (id->isEqualTo(control()->cpp11Override())) {
+                    if (_type.isOverride())
+                        translationUnit()->error(ast->specifier_token, "duplicate `override'");
+                    _type.setOverride(true);
+                }
+                else if (id->isEqualTo(control()->cpp11Final())) {
+                    if (_type.isFinal())
+                        translationUnit()->error(ast->specifier_token, "duplicate `final'");
+                    _type.setFinal(true);
+                }
+            }
+            break;
         case T_CONST:
             if (_type.isConst())
                 translationUnit()->error(ast->specifier_token, "duplicate `%s'", spell(ast->specifier_token));
@@ -2569,8 +2702,10 @@ bool Bind::visit(SimpleSpecifierAST *ast)
             break;
 
         case T_AUTO:
-            if (_type.isAuto())
-                translationUnit()->error(ast->specifier_token, "duplicate `%s'", spell(ast->specifier_token));
+            if (!translationUnit()->cxx0xEnabled()) {
+                if (_type.isAuto())
+                    translationUnit()->error(ast->specifier_token, "duplicate `%s'", spell(ast->specifier_token));
+            }
             _type.setAuto(true);
             break;
 
@@ -2640,6 +2775,18 @@ bool Bind::visit(SimpleSpecifierAST *ast)
             _type.setType(control()->integerType(IntegerType::Char));
             break;
 
+        case T_CHAR16_T:
+            if (_type)
+                translationUnit()->error(ast->specifier_token, "duplicate data type in declaration");
+            _type.setType(control()->integerType(IntegerType::Char16));
+            break;
+
+        case T_CHAR32_T:
+            if (_type)
+                translationUnit()->error(ast->specifier_token, "duplicate data type in declaration");
+            _type.setType(control()->integerType(IntegerType::Char32));
+            break;
+
         case T_WCHAR_T:
             if (_type)
                 translationUnit()->error(ast->specifier_token, "duplicate data type in declaration");
@@ -2743,6 +2890,12 @@ bool Bind::visit(TypeofSpecifierAST *ast)
     return false;
 }
 
+bool Bind::visit(DecltypeSpecifierAST *ast)
+{
+    _type = this->expression(ast->expression);
+    return false;
+}
+
 bool Bind::visit(ClassSpecifierAST *ast)
 {
     // unsigned classkey_token = ast->classkey_token;
@@ -2766,20 +2919,7 @@ bool Bind::visit(ClassSpecifierAST *ast)
             }
         }
 
-        // get the unqualified class name
-        const QualifiedNameId *q = className->asQualifiedNameId();
-        const Name *unqualifiedClassName = q ? q->name() : className;
-
-        if (! unqualifiedClassName) // paranoia check
-            className = 0;
-        else if (! (unqualifiedClassName->isNameId() || unqualifiedClassName->isTemplateNameId())) {
-            translationUnit()->error(sourceLocation, "expected a class-name");
-
-            className = unqualifiedClassName->identifier();
-
-            if (q && className)
-                className = control()->qualifiedNameId(q->base(), className);
-        }
+        ensureValidClassName(&className, sourceLocation);
     }
 
     Class *klass = control()->newClass(sourceLocation, className);
@@ -2974,6 +3114,8 @@ bool Bind::visit(FunctionDeclaratorAST *ast)
     Function *fun = control()->newFunction(0, 0);
     fun->setStartOffset(tokenAt(ast->firstToken()).begin());
     fun->setEndOffset(tokenAt(ast->lastToken() - 1).end());
+    if (ast->trailing_return_type)
+        _type = this->trailingReturnType(ast->trailing_return_type, _type);
     fun->setReturnType(_type);
 
     // unsigned lparen_token = ast->lparen_token;
@@ -2987,9 +3129,10 @@ bool Bind::visit(FunctionDeclaratorAST *ast)
     // propagate the cv-qualifiers
     fun->setConst(type.isConst());
     fun->setVolatile(type.isVolatile());
+    fun->setOverride(type.isOverride());
+    fun->setFinal(type.isFinal());
 
     this->exceptionSpecification(ast->exception_specification, type);
-    this->trailingReturnType(ast->trailing_return_type, type);
     if (ast->as_cpp_initializer != 0) {
         fun->setAmbiguous(true);
         /*ExpressionTy as_cpp_initializer =*/ this->expression(ast->as_cpp_initializer);
@@ -3007,6 +3150,23 @@ bool Bind::visit(ArrayDeclaratorAST *ast)
     return false;
 }
 
+void Bind::ensureValidClassName(const Name **name, unsigned sourceLocation)
+{
+    if (!*name)
+        return;
+
+    const QualifiedNameId *qName = (*name)->asQualifiedNameId();
+    const Name *uqName = qName ? qName->name() : *name;
+
+    if (!uqName->isNameId() && !uqName->isTemplateNameId()) {
+        translationUnit()->error(sourceLocation, "expected a class-name");
+
+        *name = uqName->identifier();
+        if (qName)
+            *name = control()->qualifiedNameId(qName->base(), *name);
+    }
+}
+
 int Bind::visibilityForAccessSpecifier(int tokenKind)
 {
     switch (tokenKind) {
