@@ -1,45 +1,42 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "wizard.h"
 
-#include <QtCore/QMap>
-#include <QtCore/QHash>
-#include <QtCore/QVariant>
+#include <QMap>
+#include <QHash>
+#include <QVariant>
 
-#include <QtGui/QLabel>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QStyle>
+#include <QLabel>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QStyle>
 
 /*! \class Utils::Wizard
 
@@ -140,20 +137,20 @@ LinearProgressWidget::LinearProgressWidget(WizardProgress *progress, QWidget *pa
     m_dotsItemWidget->setVisible(false);
     m_dotsItemWidget->setEnabled(false);
 
-    connect(m_wizardProgress, SIGNAL(itemAdded(WizardProgressItem *)),
-            this, SLOT(slotItemAdded(WizardProgressItem *)));
-    connect(m_wizardProgress, SIGNAL(itemRemoved(WizardProgressItem *)),
-            this, SLOT(slotItemRemoved(WizardProgressItem *)));
-    connect(m_wizardProgress, SIGNAL(itemChanged(WizardProgressItem *)),
-            this, SLOT(slotItemChanged(WizardProgressItem *)));
-    connect(m_wizardProgress, SIGNAL(nextItemsChanged(WizardProgressItem *, const QList<WizardProgressItem *> &)),
-            this, SLOT(slotNextItemsChanged(WizardProgressItem *, const QList<WizardProgressItem *> &)));
-    connect(m_wizardProgress, SIGNAL(nextShownItemChanged(WizardProgressItem *, WizardProgressItem *)),
-            this, SLOT(slotNextShownItemChanged(WizardProgressItem *, WizardProgressItem *)));
-    connect(m_wizardProgress, SIGNAL(startItemChanged(WizardProgressItem *)),
-            this, SLOT(slotStartItemChanged(WizardProgressItem *)));
-    connect(m_wizardProgress, SIGNAL(currentItemChanged(WizardProgressItem *)),
-            this, SLOT(slotCurrentItemChanged(WizardProgressItem *)));
+    connect(m_wizardProgress, SIGNAL(itemAdded(WizardProgressItem*)),
+            this, SLOT(slotItemAdded(WizardProgressItem*)));
+    connect(m_wizardProgress, SIGNAL(itemRemoved(WizardProgressItem*)),
+            this, SLOT(slotItemRemoved(WizardProgressItem*)));
+    connect(m_wizardProgress, SIGNAL(itemChanged(WizardProgressItem*)),
+            this, SLOT(slotItemChanged(WizardProgressItem*)));
+    connect(m_wizardProgress, SIGNAL(nextItemsChanged(WizardProgressItem*,QList<WizardProgressItem*>)),
+            this, SLOT(slotNextItemsChanged(WizardProgressItem*,QList<WizardProgressItem*>)));
+    connect(m_wizardProgress, SIGNAL(nextShownItemChanged(WizardProgressItem*,WizardProgressItem*)),
+            this, SLOT(slotNextShownItemChanged(WizardProgressItem*,WizardProgressItem*)));
+    connect(m_wizardProgress, SIGNAL(startItemChanged(WizardProgressItem*)),
+            this, SLOT(slotStartItemChanged(WizardProgressItem*)));
+    connect(m_wizardProgress, SIGNAL(currentItemChanged(WizardProgressItem*)),
+            this, SLOT(slotCurrentItemChanged(WizardProgressItem*)));
 
     QList<WizardProgressItem *> items = m_wizardProgress->items();
     for (int i = 0; i < items.count(); i++)
@@ -934,5 +931,3 @@ bool WizardProgressItem::titleWordWrap() const
 } // namespace Utils
 
 #include "wizard.moc"
-#include "moc_wizard.cpp"
-
