@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "bookmarkmanager.h"
 
@@ -39,18 +36,20 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/command.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/session.h>
 #include <texteditor/basetexteditor.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QFileInfo>
+#include <QDebug>
+#include <QDir>
+#include <QFileInfo>
 
-#include <QtGui/QAction>
-#include <QtGui/QContextMenuEvent>
-#include <QtGui/QMenu>
-#include <QtGui/QPainter>
+#include <QAction>
+#include <QContextMenuEvent>
+#include <QMenu>
+#include <QPainter>
 
 Q_DECLARE_METATYPE(Bookmarks::Internal::Bookmark*)
 
@@ -211,10 +210,10 @@ BookmarkView::BookmarkView(QWidget *parent)  :
 {
     setWindowTitle(tr("Bookmarks"));
 
-    connect(this, SIGNAL(clicked(const QModelIndex &)),
-            this, SLOT(gotoBookmark(const QModelIndex &)));
+    connect(this, SIGNAL(clicked(QModelIndex)),
+            this, SLOT(gotoBookmark(QModelIndex)));
 
-    ICore::instance()->addContextObject(m_bookmarkContext);
+    ICore::addContextObject(m_bookmarkContext);
 
     setItemDelegate(new BookmarkDelegate(this));
     setFrameStyle(QFrame::NoFrame);
@@ -224,7 +223,7 @@ BookmarkView::BookmarkView(QWidget *parent)  :
 
 BookmarkView::~BookmarkView()
 {
-    ICore::instance()->removeContextObject(m_bookmarkContext);
+    ICore::removeContextObject(m_bookmarkContext);
 }
 
 void BookmarkView::contextMenuEvent(QContextMenuEvent *event)
@@ -317,7 +316,7 @@ BookmarkManager::BookmarkManager() :
     connect(Core::ICore::instance(), SIGNAL(contextChanged(Core::IContext*,Core::Context)),
             this, SLOT(updateActionStatus()));
 
-    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(sessionLoaded()),
+    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(sessionLoaded(QString)),
             this, SLOT(loadBookmarks()));
 
     updateActionStatus();
@@ -344,7 +343,7 @@ QModelIndex BookmarkManager::index(int row, int column, const QModelIndex &paren
     if (parent.isValid())
         return QModelIndex();
     else
-        return createIndex(row, column, 0);
+        return createIndex(row, column);
 }
 
 QModelIndex BookmarkManager::parent(const QModelIndex &) const
@@ -381,7 +380,7 @@ QVariant BookmarkManager::data(const QModelIndex &index, int role) const
     else if (role == BookmarkManager::LineText)
         return m_bookmarksList.at(index.row())->lineText();
     else if (role == Qt::ToolTipRole)
-        return m_bookmarksList.at(index.row())->filePath();
+        return QDir::toNativeSeparators(m_bookmarksList.at(index.row())->filePath());
 
     return QVariant();
 }
@@ -392,7 +391,7 @@ void BookmarkManager::toggleBookmark()
     if (!editor)
         return;
 
-    toggleBookmark(editor->file()->fileName(), editor->currentLine());
+    toggleBookmark(editor->document()->fileName(), editor->currentLine());
 }
 
 void BookmarkManager::toggleBookmark(const QString &fileName, int lineNumber)
@@ -409,6 +408,7 @@ void BookmarkManager::toggleBookmark(const QString &fileName, int lineNumber)
 
     // Add a new bookmark if no bookmark existed on this line
     Bookmark *bookmark = new Bookmark(fi.filePath(), editorLine, this);
+    bookmark->init();
     addBookmark(bookmark);
 }
 
@@ -486,8 +486,10 @@ Bookmark *BookmarkManager::bookmarkForIndex(const QModelIndex &index)
 bool BookmarkManager::gotoBookmark(Bookmark *bookmark)
 {
     using namespace TextEditor;
-    if (ITextEditor *editor = BaseTextEditorWidget::openEditorAt(bookmark->filePath(), bookmark->lineNumber()))
+    if (ITextEditor *editor = qobject_cast<ITextEditor *>(BaseTextEditorWidget::openEditorAt(bookmark->filePath(),
+                                                                                             bookmark->lineNumber()))) {
         return (editor->currentLine() == bookmark->lineNumber());
+    }
     return false;
 }
 
@@ -505,7 +507,7 @@ void BookmarkManager::documentPrevNext(bool next)
 {
     TextEditor::ITextEditor *editor = currentTextEditor();
     int editorLine = editor->currentLine();
-    QFileInfo fi(editor->file()->fileName());
+    QFileInfo fi(editor->document()->fileName());
     if (!m_bookmarksMap.contains(fi.path()))
         return;
 
@@ -590,10 +592,7 @@ void BookmarkManager::prev()
 
 TextEditor::ITextEditor *BookmarkManager::currentTextEditor() const
 {
-    Core::EditorManager *em = Core::EditorManager::instance();
-    Core::IEditor *currEditor = em->currentEditor();
-    if (!currEditor)
-        return 0;
+    Core::IEditor *currEditor = EditorManager::currentEditor();
     return qobject_cast<TextEditor::ITextEditor *>(currEditor);
 }
 
@@ -612,7 +611,7 @@ BookmarkManager::State BookmarkManager::state() const
     if (!editor)
         return HasBookMarks;
 
-    const QFileInfo fi(editor->file()->fileName());
+    const QFileInfo fi(editor->document()->fileName());
 
     const DirectoryFileBookmarksMap::const_iterator dit = m_bookmarksMap.constFind(fi.path());
     if (dit == m_bookmarksMap.constEnd())
@@ -713,6 +712,7 @@ void BookmarkManager::addBookmark(const QString &s)
 
         if (!filePath.isEmpty() && !findBookmark(fi.path(), fi.fileName(), lineNumber)) {
             Bookmark *b = new Bookmark(filePath, lineNumber, this);
+            b->init();
             addBookmark(b, false);
         }
     } else {
@@ -731,33 +731,33 @@ QString BookmarkManager::bookmarkToString(const Bookmark *b)
 /* Saves the bookmarks to the session settings. */
 void BookmarkManager::saveBookmarks()
 {
-    SessionManager *s = sessionManager();
-    if (!s)
-        return;
-
     QStringList list;
     foreach (const FileNameBookmarksMap *bookmarksMap, m_bookmarksMap)
         foreach (const Bookmark *bookmark, *bookmarksMap)
             list << bookmarkToString(bookmark);
 
-    s->setValue("Bookmarks", list);
+    sessionManager()->setValue("Bookmarks", list);
 }
 
 /* Loads the bookmarks from the session settings. */
 void BookmarkManager::loadBookmarks()
 {
     removeAllBookmarks();
-    SessionManager *s = sessionManager();
-    if (!s)
-        return;
-
-    const QStringList &list = s->value("Bookmarks").toStringList();
+    const QStringList &list = sessionManager()->value("Bookmarks").toStringList();
     foreach (const QString &bookmarkString, list)
         addBookmark(bookmarkString);
 
     updateActionStatus();
 }
 
+void BookmarkManager::handleBookmarkRequest(TextEditor::ITextEditor *textEditor,
+                                            int line,
+                                            TextEditor::ITextEditor::MarkRequestKind kind)
+{
+    if (kind == TextEditor::ITextEditor::BookmarkRequest && textEditor->document())
+        toggleBookmark(textEditor->document()->fileName(), line);
+}
+
 // BookmarkViewFactory
 
 BookmarkViewFactory::BookmarkViewFactory(BookmarkManager *bm)
@@ -775,14 +775,14 @@ int BookmarkViewFactory::priority() const
     return 300;
 }
 
-QString BookmarkViewFactory::id() const
+Id BookmarkViewFactory::id() const
 {
-    return QLatin1String("Bookmarks");
+    return Id("Bookmarks");
 }
 
 QKeySequence BookmarkViewFactory::activationSequence() const
 {
-    return QKeySequence(Qt::ALT + Qt::Key_M);
+    return QKeySequence(Core::UseMacShortcuts ? tr("Alt+Meta+M") : tr("Alt+M"));
 }
 
 Core::NavigationView BookmarkViewFactory::createWidget()
