@@ -1,50 +1,47 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "fileiconprovider.h"
 #include "mimedatabase.h"
 
 #include <utils/qtcassert.h>
 
-#include <QtGui/QApplication>
-#include <QtGui/QStyle>
-#include <QtGui/QPainter>
-#include <QtCore/QFileInfo>
-#include <QtCore/QPair>
-#include <QtCore/QDebug>
+#include <QApplication>
+#include <QStyle>
+#include <QPainter>
+#include <QFileInfo>
+#include <QPair>
+#include <QDebug>
 
-#include <QtGui/QFileIconProvider>
-#include <QtGui/QIcon>
-#include <QtGui/QStyle>
+#include <QFileIconProvider>
+#include <QIcon>
+#include <QStyle>
 
 /*!
   \class Core::FileIconProvider
@@ -58,8 +55,8 @@
         own overlay icon handling (Mac/Windows).
 
   The class is a singleton: It's instance can be accessed via the static instance() method.
-  Plugins can register custom icons via registerIconSuffix(), and retrieve icons via the icon()
-  method.
+  Plugins can register custom overlay icons via registerIconOverlayForSuffix(), and
+  retrieve icons via the icon() method.
   The instance is explicitly deleted by the core plugin for destruction order reasons.
   */
 
@@ -141,7 +138,7 @@ QIcon FileIconProvider::icon(const QFileInfo &fileInfo) const
         }
     }
     // Get icon from OS.
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
     return QFileIconProvider::icon(fileInfo);
 #else
     // File icons are unknown on linux systems.
@@ -176,7 +173,7 @@ void FileIconProvider::registerIconOverlayForSuffix(const QIcon &icon,
     if (debug)
         qDebug() << "FileIconProvider::registerIconOverlayForSuffix" << suffix;
 
-    QTC_ASSERT(!icon.isNull() && !suffix.isEmpty(), return)
+    QTC_ASSERT(!icon.isNull() && !suffix.isEmpty(), return);
 
     const QPixmap fileIconPixmap = overlayIcon(QStyle::SP_FileIcon, icon, QSize(16, 16));
     // replace old icon, if it exists
