@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "outputpane.h"
 #include "outputpanemanager.h"
@@ -38,8 +35,8 @@
 #include "ioutputpane.h"
 #include "modemanager.h"
 
-#include <QtGui/QSplitter>
-#include <QtGui/QVBoxLayout>
+#include <QSplitter>
+#include <QVBoxLayout>
 
 namespace Core {
 
@@ -70,14 +67,13 @@ OutputPanePlaceHolder::OutputPanePlaceHolder(Core::IMode *mode, QSplitter* paren
     sp.setHorizontalStretch(0);
     setSizePolicy(sp);
     layout()->setMargin(0);
-    connect(Core::ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode *)),
-            this, SLOT(currentModeChanged(Core::IMode *)));
+    connect(Core::ModeManager::instance(), SIGNAL(currentModeChanged(Core::IMode*)),
+            this, SLOT(currentModeChanged(Core::IMode*)));
 }
 
 OutputPanePlaceHolder::~OutputPanePlaceHolder()
 {
     if (d->m_current == this) {
-        // FIXME: Prevent exit crash in debug mode.
         if (Internal::OutputPaneManager *om = Internal::OutputPaneManager::instance()) {
             om->setParent(0);
             om->hide();
@@ -143,6 +139,28 @@ bool OutputPanePlaceHolder::isMaximized() const
     return Internal::OutputPaneManager::instance()->isMaximized();
 }
 
+void OutputPanePlaceHolder::ensureSizeHintAsMinimum()
+{
+    if (!d->m_splitter)
+        return;
+    int idx = d->m_splitter->indexOf(this);
+    if (idx < 0)
+        return;
+
+    QList<int> sizes = d->m_splitter->sizes();
+    Internal::OutputPaneManager *om = Internal::OutputPaneManager::instance();
+    int minimum = (d->m_splitter->orientation() == Qt::Vertical
+                   ? om->sizeHint().height() : om->sizeHint().width());
+    int difference = minimum - sizes.at(idx);
+    if (difference <= 0) // is already larger
+        return;
+    for (int i = 0; i < sizes.count(); ++i) {
+        sizes[i] += difference / (sizes.count()-1);
+    }
+    sizes[idx] = minimum;
+    d->m_splitter->setSizes(sizes);
+}
+
 void OutputPanePlaceHolder::unmaximize()
 {
     if (Internal::OutputPaneManager::instance()->isMaximized())
