@@ -1,38 +1,36 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "cppelementevaluator.h"
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
+#include <cpptools/cpptoolsreuse.h>
 
 #include <FullySpecifiedType.h>
 #include <Literals.h>
@@ -41,7 +39,8 @@
 #include <Scope.h>
 #include <Symbol.h>
 #include <Symbols.h>
-#include <cplusplus/ModelManagerInterface.h>
+#include <cpptools/TypeHierarchyBuilder.h>
+#include <cpptools/ModelManagerInterface.h>
 #include <cplusplus/ExpressionUnderCursor.h>
 #include <cplusplus/Overview.h>
 #include <cplusplus/TypeOfExpression.h>
@@ -49,28 +48,16 @@
 #include <cplusplus/LookupItem.h>
 #include <cplusplus/Icons.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QFileInfo>
-#include <QtCore/QSet>
-#include <QtCore/QQueue>
+#include <QDir>
+#include <QFileInfo>
+#include <QSet>
+#include <QQueue>
 
 using namespace CppEditor;
 using namespace Internal;
 using namespace CPlusPlus;
 
 namespace {
-    void moveCursorToEndOfName(QTextCursor *tc) {
-        QTextDocument *doc = tc->document();
-        if (!doc)
-            return;
-
-        QChar ch = doc->characterAt(tc->position());
-        while (ch.isLetterOrNumber() || ch == QLatin1Char('_')) {
-            tc->movePosition(QTextCursor::NextCharacter);
-            ch = doc->characterAt(tc->position());
-        }
-    }
-
     QStringList stripName(const QString &name) {
         QStringList all;
         all << name;
@@ -88,7 +75,8 @@ CppElementEvaluator::CppElementEvaluator(CPPEditorWidget *editor) :
     m_editor(editor),
     m_modelManager(CppModelManagerInterface::instance()),
     m_tc(editor->textCursor()),
-    m_lookupBaseClasses(false)
+    m_lookupBaseClasses(false),
+    m_lookupDerivedClasses(false)
 {}
 
 void CppElementEvaluator::setTextCursor(const QTextCursor &tc)
@@ -97,6 +85,9 @@ void CppElementEvaluator::setTextCursor(const QTextCursor &tc)
 void CppElementEvaluator::setLookupBaseClasses(const bool lookup)
 { m_lookupBaseClasses = lookup; }
 
+void CppElementEvaluator::setLookupDerivedClasses(const bool lookup)
+{ m_lookupDerivedClasses = lookup; }
+
 // @todo: Consider refactoring code from CPPEditor::findLinkAt into here.
 void CppElementEvaluator::execute()
 {
@@ -106,7 +97,7 @@ void CppElementEvaluator::execute()
         return;
 
     const Snapshot &snapshot = m_modelManager->snapshot();
-    Document::Ptr doc = snapshot.document(m_editor->file()->fileName());
+    Document::Ptr doc = snapshot.document(m_editor->editorDocument()->fileName());
     if (!doc)
         return;
 
@@ -115,10 +106,10 @@ void CppElementEvaluator::execute()
     const int pos = m_tc.position();
     m_editor->convertPosition(pos, &line, &column);
 
-    checkDiagnosticMessage(doc, line);
+    checkDiagnosticMessage(pos);
 
     if (!matchIncludeFile(doc, line) && !matchMacroInUse(doc, pos)) {
-        moveCursorToEndOfName(&m_tc);
+        CppTools::moveCursorToEndOfIdentifier(&m_tc);
 
         // Fetch the expression's code
         ExpressionUnderCursor expressionUnderCursor;
@@ -127,7 +118,7 @@ void CppElementEvaluator::execute()
 
         TypeOfExpression typeOfExpression;
         typeOfExpression.init(doc, snapshot);
-        const QList<LookupItem> &lookupItems = typeOfExpression(expression, scope);
+        const QList<LookupItem> &lookupItems = typeOfExpression(expression.toUtf8(), scope);
         if (lookupItems.isEmpty())
             return;
 
@@ -136,12 +127,12 @@ void CppElementEvaluator::execute()
     }
 }
 
-void CppElementEvaluator::checkDiagnosticMessage(const CPlusPlus::Document::Ptr &document,
-                                                 unsigned line)
+void CppElementEvaluator::checkDiagnosticMessage(int pos)
 {
-    foreach (const Document::DiagnosticMessage &m, document->diagnosticMessages()) {
-        if (m.line() == line) {
-            m_diagnosis = m.text();
+    foreach (const QTextEdit::ExtraSelection &sel,
+             m_editor->extraSelections(TextEditor::BaseTextEditorWidget::CodeWarningsSelection)) {
+        if (pos >= sel.cursor.selectionStart() && pos <= sel.cursor.selectionEnd()) {
+            m_diagnosis = sel.format.toolTip();
             break;
         }
     }
@@ -163,8 +154,7 @@ bool CppElementEvaluator::matchMacroInUse(const CPlusPlus::Document::Ptr &docume
     foreach (const Document::MacroUse &use, document->macroUses()) {
         if (use.contains(pos)) {
             const unsigned begin = use.begin();
-            const QString &name = use.macro().name();
-            if (pos < begin + name.length()) {
+            if (pos < begin + use.macro().name().length()) {
                 m_element = QSharedPointer<CppElement>(new CppMacro(use.macro()));
                 return true;
             }
@@ -185,13 +175,21 @@ void CppElementEvaluator::handleLookupItemMatch(const Snapshot &snapshot,
         const FullySpecifiedType &type = declaration->type();
         if (declaration->isNamespace()) {
             m_element = QSharedPointer<CppElement>(new CppNamespace(declaration));
-        } else if (declaration->isClass() || declaration->isForwardClassDeclaration()) {
+        } else if (declaration->isClass()
+                   || declaration->isForwardClassDeclaration()
+                   || (declaration->isTemplate() && declaration->asTemplate()->declaration()
+                       && (declaration->asTemplate()->declaration()->isClass()
+                           || declaration->asTemplate()->declaration()->isForwardClassDeclaration()))) {
             if (declaration->isForwardClassDeclaration())
-                if (Symbol *classDeclaration = snapshot.findMatchingClassDeclaration(declaration))
+                if (Symbol *classDeclaration =
+                        m_symbolFinder.findMatchingClassDeclaration(declaration, snapshot)) {
                     declaration = classDeclaration;
+                }
             CppClass *cppClass = new CppClass(declaration);
             if (m_lookupBaseClasses)
                 cppClass->lookupBases(declaration, context);
+            if (m_lookupDerivedClasses)
+                cppClass->lookupDerived(declaration, snapshot);
             m_element = QSharedPointer<CppElement>(cppClass);
         } else if (Enum *enumDecl = declaration->asEnum()) {
             m_element = QSharedPointer<CppElement>(new CppEnum(enumDecl));
@@ -199,13 +197,13 @@ void CppElementEvaluator::handleLookupItemMatch(const Snapshot &snapshot,
             m_element = QSharedPointer<CppElement>(new CppEnumerator(enumerator));
         } else if (declaration->isTypedef()) {
             m_element = QSharedPointer<CppElement>(new CppTypedef(declaration));
-        } else if (declaration->isFunction() || (type.isValid() && type->isFunctionType())) {
+        } else if (declaration->isFunction()
+                   || (type.isValid() && type->isFunctionType())
+                   || declaration->isTemplate()) {
             m_element = QSharedPointer<CppElement>(new CppFunction(declaration));
         } else if (declaration->isDeclaration() && type.isValid()) {
             m_element = QSharedPointer<CppElement>(
                 new CppVariable(declaration, context, lookupItem.scope()));
-        } else if (declaration->isTemplate() && declaration->asTemplate()->declaration()) {
-            m_element = QSharedPointer<CppElement>(new CppTemplate(declaration));
         } else {
             m_element = QSharedPointer<CppElement>(new CppDeclarableElement(declaration));
         }
@@ -317,16 +315,20 @@ const QString &CppInclude::fileName() const
 CppMacro::CppMacro(const Macro &macro) : CppElement()
 {
     setHelpCategory(TextEditor::HelpItem::Macro);
-    setHelpIdCandidates(QStringList(macro.name()));
-    setHelpMark(macro.name());
+    const QString macroName = QLatin1String(macro.name());
+    setHelpIdCandidates(QStringList(macroName));
+    setHelpMark(macroName);
     setLink(CPPEditorWidget::Link(macro.fileName(), macro.line()));
-    setTooltip(macro.toString());
+    setTooltip(macro.toStringWithLineBreaks());
 }
 
 CppMacro::~CppMacro()
 {}
 
 // CppDeclarableElement
+CppDeclarableElement::CppDeclarableElement()
+{}
+
 CppDeclarableElement::CppDeclarableElement(Symbol *declaration) : CppElement()
 {
     m_icon = Icons().iconForSymbol(declaration);
@@ -388,6 +390,9 @@ CppNamespace::~CppNamespace()
 {}
 
 // CppClass
+CppClass::CppClass()
+{}
+
 CppClass::CppClass(Symbol *declaration) : CppDeclarableElement(declaration)
 {
     setHelpCategory(TextEditor::HelpItem::ClassOrNamespace);
@@ -428,9 +433,32 @@ void CppClass::lookupBases(Symbol *declaration, const CPlusPlus::LookupContext &
     }
 }
 
+void CppClass::lookupDerived(CPlusPlus::Symbol *declaration, const CPlusPlus::Snapshot &snapshot)
+{
+    typedef QPair<CppClass *, TypeHierarchy> Data;
+
+    TypeHierarchyBuilder builder(declaration, snapshot);
+    const TypeHierarchy &completeHierarchy = builder.buildDerivedTypeHierarchy();
+
+    QQueue<Data> q;
+    q.enqueue(qMakePair(this, completeHierarchy));
+    while (!q.isEmpty()) {
+        const Data &current = q.dequeue();
+        CppClass *clazz = current.first;
+        const TypeHierarchy &classHierarchy = current.second;
+        foreach (const TypeHierarchy &derivedHierarchy, classHierarchy.hierarchy()) {
+            clazz->m_derived.append(CppClass(derivedHierarchy.symbol()));
+            q.enqueue(qMakePair(&clazz->m_derived.last(), derivedHierarchy));
+        }
+    }
+}
+
 const QList<CppClass> &CppClass::bases() const
 { return m_bases; }
 
+const QList<CppClass> &CppClass::derived() const
+{ return m_derived; }
+
 // CppFunction
 CppFunction::CppFunction(Symbol *declaration) : CppDeclarableElement(declaration)
 {
@@ -515,34 +543,6 @@ CppVariable::CppVariable(Symbol *declaration, const LookupContext &context, Scop
 CppVariable::~CppVariable()
 {}
 
-// CppTemplate
-CppTemplate::CppTemplate(CPlusPlus::Symbol *declaration) : CppDeclarableElement(declaration)
-{
-    Template *templateSymbol = declaration->asTemplate();
-    if (templateSymbol->declaration()->isClass() ||
-        templateSymbol->declaration()->isForwardClassDeclaration()) {
-        m_isClassTemplate = true;
-        setHelpCategory(TextEditor::HelpItem::ClassOrNamespace);
-        setTooltip(qualifiedName());
-    } else {
-        m_isClassTemplate = false;
-        setHelpCategory(TextEditor::HelpItem::Function);
-    }
-}
-
-CppTemplate::~CppTemplate()
-{}
-
-bool CppTemplate::isClassTemplate() const
-{
-    return m_isClassTemplate;
-}
-
-bool CppTemplate::isFunctionTemplate() const
-{
-    return !m_isClassTemplate;
-}
-
 CppEnumerator::CppEnumerator(CPlusPlus::EnumeratorDeclaration *declaration)
     : CppDeclarableElement(declaration)
 {
