@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "breakhandler.h"
 #include "debuggerinternalconstants.h"
@@ -122,8 +119,8 @@ static QString typeToString(BreakpointType type)
             return BreakHandler::tr("Watchpoint at Address");
         case WatchpointAtExpression:
             return BreakHandler::tr("Watchpoint at Expression");
-        case BreakpointOnQmlSignalHandler:
-            return BreakHandler::tr("Breakpoint on QML Signal Handler");
+        case BreakpointOnQmlSignalEmit:
+            return BreakHandler::tr("Breakpoint on QML Signal Emit");
         case BreakpointAtJavaScriptThrow:
             return BreakHandler::tr("Breakpoint at JavaScript throw");
         case UnknownType:
@@ -323,6 +320,8 @@ void BreakHandler::saveBreakpoints()
             map.insert(_("threadspec"), data.threadSpec);
         if (!data.enabled)
             map.insert(_("disabled"), one);
+        if (data.oneShot)
+            map.insert(_("oneshot"), one);
         if (data.pathUsage != BreakpointPathUsageEngineDefault)
             map.insert(_("usefullpath"), QString::number(data.pathUsage));
         if (data.tracepoint)
@@ -375,6 +374,9 @@ void BreakHandler::loadBreakpoints()
         v = map.value(_("disabled"));
         if (v.isValid())
             data.enabled = !v.toInt();
+        v = map.value(_("oneshot"));
+        if (v.isValid())
+            data.oneShot = v.toInt();
         v = map.value(_("usefullpath"));
         if (v.isValid())
             data.pathUsage = static_cast<BreakpointPathUsage>(v.toInt());
@@ -770,6 +772,13 @@ bool BreakHandler::isTracepoint(BreakpointModelId id) const
     return it->data.tracepoint;
 }
 
+bool BreakHandler::isOneShot(BreakpointModelId id) const
+{
+    ConstIterator it = m_storage.find(id);
+    BREAK_ASSERT(it != m_storage.end(), return false);
+    return it->data.oneShot;
+}
+
 bool BreakHandler::needsChildren(BreakpointModelId id) const
 {
     ConstIterator it = m_storage.find(id);
@@ -1569,5 +1578,32 @@ QString BreakHandler::BreakpointItem::toToolTip() const
     return rc;
 }
 
+void BreakHandler::setWatchpointAtAddress(quint64 address, unsigned size)
+{
+    BreakpointParameters data(WatchpointAtAddress);
+    data.address = address;
+    data.size = size;
+    BreakpointModelId id = findWatchpoint(data);
+    if (id) {
+        qDebug() << "WATCHPOINT EXISTS";
+        //   removeBreakpoint(index);
+        return;
+    }
+    appendBreakpoint(data);
+}
+
+void BreakHandler::setWatchpointAtExpression(const QString &exp)
+{
+    BreakpointParameters data(WatchpointAtExpression);
+    data.expression = exp;
+    BreakpointModelId id = findWatchpoint(data);
+    if (id) {
+        qDebug() << "WATCHPOINT EXISTS";
+        //   removeBreakpoint(index);
+        return;
+    }
+    appendBreakpoint(data);
+}
+
 } // namespace Internal
 } // namespace Debugger
