@@ -1,36 +1,34 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2012 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "debuggertooltipmanager.h"
+#include "debuggerinternalconstants.h"
 #include "watchutils.h"
 #include "debuggerengine.h"
 #include "debuggeractions.h"
@@ -38,7 +36,6 @@
 #include "watchutils.h"
 #include "stackhandler.h"
 #include "debuggercore.h"
-#include "debuggerinternalconstants.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/modemanager.h>
@@ -58,7 +55,6 @@
 #include <QStyle>
 #include <QIcon>
 #include <QApplication>
-#include <QMainWindow>
 #include <QMoveEvent>
 #include <QDesktopWidget>
 #include <QScrollBar>
@@ -101,8 +97,8 @@ static const char offsetYAttributeC[] = "offset_y";
 static const char engineTypeAttributeC[] = "engine";
 static const char dateAttributeC[] = "date";
 static const char treeElementC[] = "tree";
-static const char treeModelAttributeC[] = "model"; // Locals/Watches
-static const char treeExpressionAttributeC[] = "expression"; // Locals/Watches
+static const char treeExpressionAttributeC[] = "expression";
+static const char treeInameAttributeC[] = "iname";
 static const char modelElementC[] = "model";
 static const char modelColumnCountAttributeC[] = "columncount";
 static const char modelRowElementC[] = "row";
@@ -237,7 +233,7 @@ DebuggerToolTipEditor::DebuggerToolTipEditor(IEditor *ie) :
 
 DebuggerToolTipEditor DebuggerToolTipEditor::currentToolTipEditor()
 {
-    if (IEditor *ie = EditorManager::instance()->currentEditor())
+    if (IEditor *ie = EditorManager::currentEditor())
         return DebuggerToolTipEditor(ie);
     return DebuggerToolTipEditor();
 }
@@ -579,7 +575,7 @@ QDebug operator<<(QDebug d, const DebuggerToolTipContext &c)
     It consists of a title toolbar and a vertical main layout.
     The widget has the ability to save/restore tree model contents to XML.
     With the engine acquired, it sets a filter model (by expression) on
-    the engine's Locals model.
+    one of the engine's models (debuggerModel).
     On release, it serializes and restores the data to a QStandardItemModel
     (defaultModel) and displays that.
 
@@ -651,8 +647,8 @@ DebuggerToolTipWidget::DebuggerToolTipWidget(QWidget *parent) :
 }
 
 bool DebuggerToolTipWidget::matches(const QString &fileName,
-                                    const QString &engineType,
-                                    const QString &function) const
+                                            const QString &engineType,
+                                            const QString &function) const
 {
     if (fileName.isEmpty() || m_context.fileName != fileName)
         return false;
@@ -664,7 +660,7 @@ bool DebuggerToolTipWidget::matches(const QString &fileName,
     return function == m_context.function;
 }
 
-void DebuggerToolTipWidget::acquireEngine(Debugger::DebuggerEngine *engine)
+void DebuggerToolTipWidget::acquireEngine(DebuggerEngine *engine)
 {
     QTC_ASSERT(engine, return);
 
@@ -694,9 +690,7 @@ void DebuggerToolTipWidget::copy()
 {
     const QString clipboardText = clipboardContents();
     QClipboard *clipboard = QApplication::clipboard();
-#ifdef Q_WS_X11
     clipboard->setText(clipboardText, QClipboard::Selection);
-#endif
     clipboard->setText(clipboardText, QClipboard::Clipboard);
 }
 
@@ -836,61 +830,46 @@ void DebuggerToolTipWidget::saveSessionData(QXmlStreamWriter &w) const
 }
 
 /*!
-    \class Debugger::Internal::DebuggerToolTipExpressionFilterModel
+    \class Debugger::Internal::TooltipFilterModel
 
-    \brief Model for tooltips filtering a local variable using the locals or tooltip model,
-    matching on the name.
-
-    Expressions/names can either be flat ('foo' will match at the root level)
-    or nested ('this.m_foo' will match 'this' at root level and 'm_foo' at level 1).
+    \brief Model for tooltips filtering an item on the watchhandler matching its tree on the iname.
 
     In addition, suppress the model's tooltip data to avoid a tooltip on a tooltip.
 */
 
-class DebuggerToolTipExpressionFilterModel : public QSortFilterProxyModel
+class TooltipFilterModel : public QSortFilterProxyModel
 {
 public:
-    explicit DebuggerToolTipExpressionFilterModel(QAbstractItemModel *model, const QString &exp, QObject *parent = 0);
-    virtual bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
+    TooltipFilterModel(QAbstractItemModel *model, const QByteArray &iname)
+        : m_iname(iname)
+    {
+        setSourceModel(model);
+    }
 
-    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const
+    {
+        return role == Qt::ToolTipRole
+            ? QVariant() : QSortFilterProxyModel::data(index, role);
+    }
+
+    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
 
 private:
-    const QStringList m_expressions;
+    const QByteArray m_iname;
 };
 
-DebuggerToolTipExpressionFilterModel::DebuggerToolTipExpressionFilterModel(QAbstractItemModel *model,
-                                                                           const QString &exp,
-                                                                           QObject *parent) :
-    QSortFilterProxyModel(parent),
-    m_expressions(exp.split(QLatin1Char('.')))
+static bool isSubIname(const QByteArray &haystack, const QByteArray &needle)
 {
-    setSourceModel(model);
+    return haystack.size() > needle.size()
+           && haystack.startsWith(needle)
+           && haystack.at(needle.size()) == '.';
 }
 
-QVariant DebuggerToolTipExpressionFilterModel::data(const QModelIndex &index, int role) const
+bool TooltipFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
-    return role != Qt::ToolTipRole ?
-        QSortFilterProxyModel::data(index, role) : QVariant();
-}
-
-// Return depth of a model index, that is, 0 for root index, 1 for level-1 children, etc.
-static inline int indexDepth(QModelIndex index)
-{
-    int depth = 0;
-    for ( ; index.isValid() ; index = index.parent())
-        depth++;
-    return depth;
-}
-
-bool DebuggerToolTipExpressionFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
-{
-    // Match on expression for top level, else pass through.
-    const int depth = indexDepth(sourceParent);
-    if (depth >= m_expressions.size()) // No filters at this level
-        return true;
     const QModelIndex nameIndex = sourceModel()->index(sourceRow, 0, sourceParent);
-    return nameIndex.data().toString() == m_expressions.at(depth);
+    const QByteArray iname = nameIndex.data(LocalsINameRole).toByteArray();
+    return iname == m_iname || isSubIname(iname, m_iname) || isSubIname(m_iname, iname);
 }
 
 /*!
@@ -924,6 +903,7 @@ QAbstractItemModel *DebuggerToolTipTreeView::swapModel(QAbstractItemModel *newMo
         if (previousModel)
             previousModel->disconnect(SIGNAL(rowsInserted(QModelIndex,int,int)), this);
         setModel(newModel);
+        //setRootIndex(newModel->index(0, 0));
         connect(newModel, SIGNAL(rowsInserted(QModelIndex,int,int)),
                 this, SLOT(computeSize()), Qt::QueuedConnection);
         computeSize();
@@ -991,13 +971,12 @@ void DebuggerToolTipTreeView::computeSize()
     setRootIsDecorated(rootDecorated);
 }
 
-void DebuggerToolTipWidget::doAcquireEngine(Debugger::DebuggerEngine *engine)
+void DebuggerToolTipWidget::doAcquireEngine(DebuggerEngine *engine)
 {
     // Create a filter model on the debugger's model and switch to it.
-    QAbstractItemModel *model = engine->localsModel();
-    QTC_ASSERT(model, return);
-    DebuggerToolTipExpressionFilterModel *filterModel =
-            new DebuggerToolTipExpressionFilterModel(model, m_expression);
+    QAbstractItemModel *model = engine->watchModel();
+    TooltipFilterModel *filterModel =
+            new TooltipFilterModel(model, m_iname);
     swapModel(filterModel);
 }
 
@@ -1006,7 +985,7 @@ QAbstractItemModel *DebuggerToolTipWidget::swapModel(QAbstractItemModel *newMode
     QAbstractItemModel *oldModel = m_treeView->swapModel(newModel);
     // When looking at some 'this.m_foo.x', expand all items
     if (newModel) {
-        if (const int level = m_expression.count(QLatin1Char('.')) + 1) {
+        if (const int level = m_iname.count('.')) {
             QModelIndex index = newModel->index(0, 0);
             for (int i = 0; i < level && index.isValid(); i++, index = index.child(0, 0))
                 m_treeView->setExpanded(index, true);
@@ -1068,7 +1047,9 @@ void DebuggerToolTipWidget::doSaveSessionData(QXmlStreamWriter &w) const
 {
     w.writeStartElement(QLatin1String(treeElementC));
     QXmlStreamAttributes attributes;
-    attributes.append(QLatin1String(treeExpressionAttributeC), m_expression);
+    if (!m_expression.isEmpty())
+        attributes.append(QLatin1String(treeExpressionAttributeC), m_expression);
+    attributes.append(QLatin1String(treeInameAttributeC), QLatin1String(m_iname));
     w.writeAttributes(attributes);
     if (QAbstractItemModel *model = m_treeView->model()) {
         XmlWriterTreeModelVisitor v(model, w);
@@ -1083,10 +1064,11 @@ void DebuggerToolTipWidget::doLoadSessionData(QXmlStreamReader &r)
         return;
     // Restore data to default model and show that.
     const QXmlStreamAttributes attributes = r.attributes();
+    m_iname = attributes.value(QLatin1String(treeInameAttributeC)).toString().toLatin1();
     m_expression = attributes.value(QLatin1String(treeExpressionAttributeC)).toString();
     if (debugToolTips)
-        qDebug() << "DebuggerTreeViewToolTipWidget::doLoadSessionData() "  << m_expression;
-    setObjectName(QLatin1String("DebuggerTreeViewToolTipWidget: ") + m_expression);
+        qDebug() << "DebuggerTreeViewToolTipWidget::doLoadSessionData() " << m_debuggerModel << m_iname;
+    setObjectName(QLatin1String("DebuggerTreeViewToolTipWidget: ") + QLatin1String(m_iname));
     restoreTreeModel(r, m_defaultModel);
     r.readNext(); // Skip </tree>
     m_treeView->swapModel(m_defaultModel);
@@ -1298,7 +1280,7 @@ void DebuggerToolTipManager::slotUpdateVisibleToolTips()
     }
 }
 
-void DebuggerToolTipManager::slotDebuggerStateChanged(Debugger::DebuggerState state)
+void DebuggerToolTipManager::slotDebuggerStateChanged(DebuggerState state)
 {
     const QObject *engine = sender();
     QTC_ASSERT(engine, return);
@@ -1306,7 +1288,7 @@ void DebuggerToolTipManager::slotDebuggerStateChanged(Debugger::DebuggerState st
     const QString name = engine->objectName();
     if (debugToolTips)
         qDebug() << "DebuggerToolTipWidget::debuggerStateChanged"
-                 << engine << Debugger::DebuggerEngine::stateName(state);
+                 << engine << DebuggerEngine::stateName(state);
 
     // Release at earliest possible convenience.
     switch (state) {
@@ -1484,14 +1466,16 @@ void DebuggerToolTipManager::slotTooltipOverrideRequested(ITextEditor *editor,
         qDebug() << "<slotTooltipOverrideRequested() " << currentEngine << *handled;
 }
 
-QStringList DebuggerToolTipManager::treeWidgetExpressions(const QString &fileName,
-                                                          const QString &engineType,
-                                                          const QString &function) const
+
+DebuggerToolTipManager::ExpressionInamePairs
+    DebuggerToolTipManager::treeWidgetExpressions(const QString &fileName,
+                                                  const QString &engineType,
+                                                  const QString &function) const
 {
-    QStringList rc;
+    ExpressionInamePairs rc;
     foreach (const QPointer<DebuggerToolTipWidget> &tw, m_tooltips) {
         if (!tw.isNull() && tw->matches(fileName, engineType, function))
-            rc.push_back(tw->expression());
+            rc.push_back(ExpressionInamePair(tw->expression(), tw->iname()));
     }
     if (debugToolTips)
         qDebug() << "DebuggerToolTipManager::treeWidgetExpressions"
