@@ -1,40 +1,36 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "gdbengine.h"
 #include "gdbmi.h"
 
 #include "debuggerstartparameters.h"
-#include "abstractgdbadapter.h"
 #include "debuggeractions.h"
 #include "debuggercore.h"
 #include "debuggerstringutils.h"
@@ -42,18 +38,22 @@
 #include "stackhandler.h"
 #include "watchhandler.h"
 
+#include <coreplugin/icore.h>
+#include <qtsupport/qtsupportconstants.h>
 #include <utils/qtcassert.h>
 #include <utils/savedaction.h>
 #include <utils/fileutils.h>
 
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
-#include <QtGui/QMessageBox>
+#include <QFile>
+#include <QFileInfo>
+#include <QMessageBox>
+#include <QPushButton>
 
 #if !defined(Q_OS_WIN)
 #include <dlfcn.h>
 #endif
 
+#include <cctype>
 
 #define PRECONDITION QTC_CHECK(!hasPython())
 
@@ -112,6 +112,27 @@ QString DumperHelper::msgDumperOutdated(double requiredVersion, double currentVe
        arg(currentVersion).arg(requiredVersion);
 }
 
+QString DumperHelper::msgPtraceError(DebuggerStartMode sm)
+{
+    if (sm == StartInternal) {
+        return QCoreApplication::translate("QtDumperHelper",
+            "ptrace: Operation not permitted.\n\n"
+            "Could not attach to the process. "
+            "Make sure no other debugger traces this process.\n"
+            "Check the settings of\n"
+            "/proc/sys/kernel/yama/ptrace_scope\n"
+            "For more details, see /etc/sysctl.d/10-ptrace.conf\n");
+    }
+    return QCoreApplication::translate("QtDumperHelper",
+        "ptrace: Operation not permitted.\n\n"
+        "Could not attach to the process. "
+        "Make sure no other debugger traces this process.\n"
+        "If your uid matches the uid\n"
+        "of the target process, check the settings of\n"
+        "/proc/sys/kernel/yama/ptrace_scope\n"
+        "For more details, see /etc/sysctl.d/10-ptrace.conf\n");
+}
+
 static inline void formatQtVersion(int v, QTextStream &str)
 {
     str  << ((v >> 16) & 0xFF) << '.' << ((v >> 8) & 0xFF) << '.' << (v & 0xFF);
@@ -144,11 +165,11 @@ QString DumperHelper::toString(bool debug) const
         return rc;
     }
     const QString nameSpace = m_qtNamespace.isEmpty()
-        ? QCoreApplication::translate("QtDumperHelper", "<none>") : m_qtNamespace;
+        ? QCoreApplication::translate("QtDumperHelper", "<none>") : QLatin1String(m_qtNamespace);
     return QCoreApplication::translate("QtDumperHelper",
        "%n known types, Qt version: %1, Qt namespace: %2 Dumper version: %3",
        0, QCoreApplication::CodecForTr,
-       m_nameTypeMap.size()).arg(qtVersionString(), nameSpace).arg(m_dumperVersion);
+       m_nameTypeMap.size()).arg(QLatin1String(qtVersionString()), nameSpace).arg(m_dumperVersion);
 }
 
 DumperHelper::Type DumperHelper::simpleType(const QByteArray &simpleType) const
@@ -256,7 +277,7 @@ static double getDumperVersion(const GdbMi &contents)
     const GdbMi dumperVersionG = contents.findChild("dumperversion");
     if (dumperVersionG.type() != GdbMi::Invalid) {
         bool ok;
-        const double v = QString::fromAscii(dumperVersionG.data()).toDouble(&ok);
+        const double v = QString::fromLatin1(dumperVersionG.data()).toDouble(&ok);
         if (ok)
             return v;
     }
@@ -475,13 +496,13 @@ DumperHelper::SpecialSizeType DumperHelper::specialSizeType(const QByteArray &ty
     return SpecialSizeCount;
 }
 
-static inline bool isInteger(const QString &n)
+static inline bool isInteger(const QByteArray &n)
 {
     const int size = n.size();
     if (!size)
         return false;
     for (int i = 0; i < size; i++)
-        if (!n.at(i).isDigit())
+        if (!std::isdigit(n.at(i)))
             return false;
     return true;
 }
@@ -506,12 +527,12 @@ void DumperHelper::evaluationParameters(const WatchData &data,
     for (int i = 0; i != inners.size(); ++i)
         inners[i] = inners[i].simplified();
 
-    QString outertype = td.isTemplate ? td.tmplate : data.type;
+    QByteArray outertype = td.isTemplate ? td.tmplate : data.type;
     // adjust the data extract
     if (outertype == m_qtNamespace + "QWidget")
         outertype = m_qtNamespace + "QObject";
 
-    QString inner = td.inner;
+    QByteArray inner = td.inner;
     const QByteArray zero = "0";
 
     extraArgs.clear();
@@ -644,13 +665,13 @@ void DumperHelper::evaluationParameters(const WatchData &data,
     }
 
     inBuffer->clear();
-    inBuffer->append(outertype.toUtf8());
+    inBuffer->append(outertype);
     inBuffer->append('\0');
     inBuffer->append(data.iname);
     inBuffer->append('\0');
     inBuffer->append(data.exp);
     inBuffer->append('\0');
-    inBuffer->append(inner.toUtf8());
+    inBuffer->append(inner);
     inBuffer->append('\0');
     inBuffer->append(data.iname);
     inBuffer->append('\0');
@@ -697,23 +718,17 @@ static bool parseConsoleStream(const GdbResponse &response, GdbMi *contents)
 void GdbEngine::updateLocalsClassic()
 {
     PRECONDITION;
-    m_pendingWatchRequests = 0;
+    //m_pendingWatchRequests = 0;
     m_pendingBreakpointRequests = 0;
     m_processedNames.clear();
 
-    if (0 && debugPending)
-        qDebug() << "\nRESET PENDING";
-    //m_toolTipCache.clear();
-    clearToolTip();
-    watchHandler()->beginCycle();
-
     QByteArray level = QByteArray::number(currentFrame());
     // '2' is 'list with type and value'
     QByteArray cmd = "-stack-list-arguments 2 " + level + ' ' + level;
-    postCommand(cmd, WatchUpdate,
+    postCommand(cmd, Discardable,
         CB(handleStackListArgumentsClassic));
     // '2' is 'list with type and value'
-    postCommand("-stack-list-locals 2", WatchUpdate,
+    postCommand("-stack-list-locals 2", Discardable,
         CB(handleStackListLocalsClassic)); // stage 2/2
 }
 
@@ -735,9 +750,9 @@ void GdbEngine::runDirectDebuggingHelperClassic(const WatchData &data, bool dump
 
     QVariant var;
     var.setValue(data);
-    postCommand(cmd, WatchUpdate, CB(handleDebuggingHelperValue3Classic), var);
+    postCommand(cmd, Discardable, CB(handleDebuggingHelperValue3Classic), var);
 
-    showStatusMessage(msgRetrievingWatchData(m_pendingWatchRequests + 1), 10000);
+    showStatusMessage(msgRetrievingWatchData(m_uncompleted.size()), 10000);
 }
 
 void GdbEngine::runDebuggingHelperClassic(const WatchData &data0, bool dumpChildren)
@@ -792,29 +807,25 @@ void GdbEngine::runDebuggingHelperClassic(const WatchData &data0, bool dumpChild
         cmd += ',' + ex;
     cmd += ')';
 
-    postCommand(cmd, WatchUpdate | NonCriticalResponse);
+    postCommand(cmd, Discardable | NonCriticalResponse);
 
-    showStatusMessage(msgRetrievingWatchData(m_pendingWatchRequests + 1), 10000);
+    showStatusMessage(msgRetrievingWatchData(m_uncompleted.size()), 10000);
 
     // retrieve response
-    postCommand("p (char*)&qDumpOutBuffer", WatchUpdate,
+    postCommand("p (char*)&qDumpOutBuffer", Discardable,
         CB(handleDebuggingHelperValue2Classic), qVariantFromValue(data));
 }
 
 void GdbEngine::createGdbVariableClassic(const WatchData &data)
 {
     PRECONDITION;
-    if (data.iname == "local.flist.0") {
-        int i = 1;
-        Q_UNUSED(i);
-    }
-    postCommand("-var-delete \"" + data.iname + '"', WatchUpdate);
+    postCommand("-var-delete \"" + data.iname + '"', Discardable);
     QByteArray exp = data.exp;
     if (exp.isEmpty() && data.address)
         exp = "*(" + gdbQuoteTypes(data.type) + "*)" + data.hexAddress();
     QVariant val = QVariant::fromValue<WatchData>(data);
     postCommand("-var-create \"" + data.iname + "\" * \"" + exp + '"',
-        WatchUpdate, CB(handleVarCreate), val);
+        Discardable, CB(handleVarCreate), val);
 }
 
 void GdbEngine::updateSubItemClassic(const WatchData &data0)
@@ -864,8 +875,8 @@ void GdbEngine::updateSubItemClassic(const WatchData &data0)
             // Try automatic dereferentiation
             data.exp = "(*(" + data.exp + "))";
             data.type = data.type + '.'; // FIXME: fragile HACK to avoid recursion
-            if (data.value.startsWith("0x"))
-                data.value = "@" + data.value;
+            if (data.value.startsWith(QLatin1String("0x")))
+                data.value.insert(0, QLatin1Char('@'));
             insertData(data);
         } else {
             data.setChildrenUnneeded();
@@ -914,7 +925,7 @@ void GdbEngine::updateSubItemClassic(const WatchData &data0)
         if (debugSubItem)
             qDebug() << "UPDATE SUBITEM: VALUE";
         QByteArray cmd = "-var-evaluate-expression \"" + data.iname + '"';
-        postCommand(cmd, WatchUpdate,
+        postCommand(cmd, Discardable,
             CB(handleEvaluateExpressionClassic), QVariant::fromValue(data));
         return;
     }
@@ -938,7 +949,7 @@ void GdbEngine::updateSubItemClassic(const WatchData &data0)
     if (data.isChildrenNeeded()) {
         QTC_ASSERT(!data.variable.isEmpty(), return); // tested above
         QByteArray cmd = "-var-list-children --all-values \"" + data.variable + '"';
-        postCommand(cmd, WatchUpdate,
+        postCommand(cmd, Discardable,
             CB(handleVarListChildrenClassic), QVariant::fromValue(data));
         return;
     }
@@ -984,7 +995,7 @@ void GdbEngine::handleDebuggingHelperValue2Classic(const GdbResponse &response)
     if (m_cookieForToken.contains(response.token - 1)) {
         m_cookieForToken.remove(response.token - 1);
         showMessage(_("DETECTING LOST COMMAND %1").arg(response.token - 1));
-        --m_pendingWatchRequests;
+        // --m_pendingWatchRequests;
         data.setError(WatchData::msgNotInScope());
         insertData(data);
         return;
@@ -1010,7 +1021,8 @@ void GdbEngine::handleDebuggingHelperValue2Classic(const GdbResponse &response)
     parseWatchData(watchHandler()->expandedINames(), data, contents, &list);
     //for (int i = 0; i != list.size(); ++i)
     //    qDebug() << "READ: " << list.at(i).toString();
-    watchHandler()->insertBulkData(list);
+    foreach (const WatchData &data, list)
+        insertData(data);
 }
 
 void GdbEngine::handleDebuggingHelperValue3Classic(const GdbResponse &response)
@@ -1032,7 +1044,7 @@ void GdbEngine::handleDebuggingHelperValue3Classic(const GdbResponse &response)
             int l = out.isEmpty() ? 0 : list.size();
             for (int i = 0; i < l; ++i)
                  str.append(list.at(i).toInt());
-            data.setValue(_c('"') + str + _c('"'));
+            data.setValue(QLatin1Char('"') + str + QLatin1Char('"'));
             data.setHasChildren(false);
             data.setAllUnneeded();
             insertData(data);
@@ -1067,7 +1079,7 @@ void GdbEngine::handleDebuggingHelperValue3Classic(const GdbResponse &response)
                     QByteArray cmd = "qdumpqstring (" + data1.exp + ')';
                     QVariant var;
                     var.setValue(data1);
-                    postCommand(cmd, WatchUpdate,
+                    postCommand(cmd, Discardable,
                         CB(handleDebuggingHelperValue3Classic), var);
                 }
             }
@@ -1086,8 +1098,11 @@ void GdbEngine::handleDebuggingHelperValue3Classic(const GdbResponse &response)
 
 void GdbEngine::tryLoadDebuggingHelpersClassic()
 {
+    if (m_forceAsyncModel)
+        return;
+
     PRECONDITION;
-    if (m_gdbAdapter->dumperHandling() == AbstractGdbAdapter::DumperNotAvailable) {
+    if (dumperHandling() == GdbEngine::DumperNotAvailable) {
         // Load at least gdb macro based dumpers.
         m_debuggingHelperState = DebuggingHelperLoadTried;
         postCommand(Utils::FileReader::fetchQrc(_(":/gdb/gdbmacros.txt")));
@@ -1101,18 +1116,13 @@ void GdbEngine::tryLoadDebuggingHelpersClassic()
         return;
 
     m_debuggingHelperState = DebuggingHelperLoadTried;
-    QByteArray dlopenLib;
-    const DebuggerStartMode startMode = startParameters().startMode;
-    if (startMode == AttachToRemote || startMode == StartRemoteGdb)
-        dlopenLib = startParameters().remoteDumperLib;
-    else
-        dlopenLib = qtDumperLibraryName().toLocal8Bit();
 
     // Do not use STRINGIFY for RTLD_NOW as we really want to expand that to a number.
-#if defined(Q_OS_WIN) || defined(Q_OS_SYMBIAN)
-    // We are using Python on Windows and Symbian.
+#if defined(Q_OS_WIN)
+    // We are using Python on Windows.
     QTC_CHECK(false);
 #elif defined(Q_OS_MAC)
+    QByteArray dlopenLib = startParameters().dumperLibrary.toLocal8Bit();
     //postCommand("sharedlibrary libc"); // for malloc
     //postCommand("sharedlibrary libdl"); // for dlopen
     const QByteArray flag = QByteArray::number(RTLD_NOW);
@@ -1121,6 +1131,7 @@ void GdbEngine::tryLoadDebuggingHelpersClassic()
         CB(handleDebuggingHelperSetup));
     //postCommand("sharedlibrary " + dotEscape(dlopenLib));
 #else
+    QByteArray dlopenLib = startParameters().dumperLibrary.toLocal8Bit();
     //postCommand("p dlopen");
     const QByteArray flag = QByteArray::number(RTLD_NOW);
     postCommand("sharedlibrary libc"); // for malloc
@@ -1134,16 +1145,10 @@ void GdbEngine::tryLoadDebuggingHelpersClassic()
         CB(handleDebuggingHelperSetup));
     postCommand("sharedlibrary " + dotEscape(dlopenLib));
 #endif
-    tryQueryDebuggingHelpersClassic();
-}
 
-void GdbEngine::tryQueryDebuggingHelpersClassic()
-{
-    PRECONDITION;
     // Retrieve list of dumpable classes.
     postCommand("call (void*)qDumpObjectData440(1,0,0,0,0,0,0,0)");
-    postCommand("p (char*)&qDumpOutBuffer",
-        CB(handleQueryDebuggingHelperClassic));
+    postCommand("p (char*)&qDumpOutBuffer", CB(handleQueryDebuggingHelperClassic));
 }
 
 // Called from CoreAdapter and AttachAdapter
@@ -1156,12 +1161,12 @@ void GdbEngine::updateAllClassic()
         qDebug() << state());
     tryLoadDebuggingHelpersClassic();
     reloadModulesInternal();
-    postCommand("-stack-list-frames", WatchUpdate,
+    postCommand("-stack-list-frames", Discardable,
         CB(handleStackListFrames),
         QVariant::fromValue<StackCookie>(StackCookie(false, true)));
     stackHandler()->setCurrentIndex(0);
     if (supportsThreads())
-        postCommand("-thread-list-ids", WatchUpdate, CB(handleThreadListIds), 0);
+        postCommand("-thread-list-ids", Discardable, CB(handleThreadListIds), 0);
     reloadRegisters();
     updateLocals();
 }
@@ -1205,7 +1210,7 @@ void GdbEngine::handleStackListArgumentsClassic(const GdbResponse &response)
     } else {
         // Seems to occur on "RedHat 4 based Linux" gdb 7.0.1:
         // ^error,msg="Cannot access memory at address 0x0"
-        showMessage(_("UNEXPECTED RESPONSE: ") + response.toString());
+        showMessage(_("UNEXPECTED RESPONSE: ") + QLatin1String(response.toString()));
     }
 }
 
@@ -1233,34 +1238,68 @@ void GdbEngine::handleStackListLocalsClassic(const GdbResponse &response)
                                       frame.function, frame.file, frame.line,
                                       &uninitializedVariables);
     }
-    QList<WatchData> list;
+    WatchHandler *handler = watchHandler();
+    insertData(*handler->findData("local"));
+
     foreach (const GdbMi &item, locals) {
         const WatchData data = localVariable(item, uninitializedVariables, &seen);
         if (data.isValid())
-            list.push_back(data);
+            insertData(data);
     }
 
     if (!m_resultVarName.isEmpty()) {
         WatchData rd;
         rd.iname = "return.0";
-        rd.name = "return";
+        rd.name = QLatin1String("return");
         rd.exp = m_resultVarName;
-        list.append(rd);
+        insertData(rd);
     }
 
-    watchHandler()->insertBulkData(list);
-    watchHandler()->updateWatchers();
+    handler->updateWatchers();
+}
+
+static void showQtDumperLibraryWarning(const QString &details)
+{
+    QMessageBox dialog(debuggerCore()->mainWindow());
+    QPushButton *qtPref = dialog.addButton(DebuggerCore::tr("Open Qt Options"),
+        QMessageBox::ActionRole);
+    QPushButton *helperOff = dialog.addButton(DebuggerCore::tr("Turn off Helper Usage"),
+        QMessageBox::ActionRole);
+    QPushButton *justContinue = dialog.addButton(DebuggerCore::tr("Continue Anyway"),
+        QMessageBox::AcceptRole);
+    dialog.setDefaultButton(justContinue);
+    dialog.setWindowTitle(DebuggerCore::tr("Debugging Helper Missing"));
+    dialog.setText(DebuggerCore::tr("The debugger could not load the debugging helper library."));
+    dialog.setInformativeText(DebuggerCore::tr(
+        "The debugging helper is used to nicely format the values of some Qt "
+        "and Standard Library data types. "
+        "It must be compiled for each used Qt version separately. "
+        "In the Qt Creator Build and Run preferences page, select a Qt version, "
+        "expand the Details section and click Build All."));
+    if (!details.isEmpty())
+        dialog.setDetailedText(details);
+#if defined(Q_OS_MAC) && QT_VERSION >= 0x050000
+    dialog.setWindowModality(Qt::WindowModal);
+#endif
+    dialog.exec();
+    if (dialog.clickedButton() == qtPref) {
+        Core::ICore::showOptionsDialog(
+            _(ProjectExplorer::Constants::PROJECTEXPLORER_SETTINGS_CATEGORY),
+            _(QtSupport::Constants::QTVERSION_SETTINGS_PAGE_ID));
+    } else if (dialog.clickedButton() == helperOff) {
+        debuggerCore()->action(UseDebuggingHelpers)->setValue(qVariantFromValue(false), false);
+    }
 }
 
 bool GdbEngine::checkDebuggingHelpersClassic()
 {
     PRECONDITION;
-    if (!qtDumperLibraryEnabled())
+    if (!debuggerCore()->boolSetting(UseDebuggingHelpers))
         return false;
-    const QString lib = qtDumperLibraryName();
+    const QString lib = startParameters().dumperLibrary;
     if (QFileInfo(lib).exists())
         return true;
-    const QStringList &locations = qtDumperLibraryLocations();
+    const QStringList &locations = startParameters().dumperLibraryLocations;
     const QString loc = locations.join(QLatin1String(", "));
     const QString msg = tr("The debugging helper library was not found at %1.")
             .arg(loc);
@@ -1314,10 +1353,10 @@ void GdbEngine::handleDebuggingHelperVersionCheckClassic(const GdbResponse &resp
     if (response.resultClass == GdbResultDone) {
         QString value = _(response.data.findChild("value").data());
         QString debuggeeQtVersion = value.section(QLatin1Char('"'), 1, 1);
-        QString dumperQtVersion = m_dumperHelper.qtVersionString();
+        QString dumperQtVersion = QLatin1String(m_dumperHelper.qtVersionString());
         if (debuggeeQtVersion.isEmpty()) {
             showMessage(_("DUMPER VERSION CHECK SKIPPED, NO qVersion() OUTPUT IN")
-                + response.toString());
+                        + QLatin1String(response.toString()));
         } else if (dumperQtVersion.isEmpty()) {
             showMessage(_("DUMPER VERSION CHECK SKIPPED, NO VERSION STRING"));
         } else if (dumperQtVersion != debuggeeQtVersion) {
@@ -1332,7 +1371,7 @@ void GdbEngine::handleDebuggingHelperVersionCheckClassic(const GdbResponse &resp
                          + dumperQtVersion);
         }
     } else {
-        showMessage("DUMPER VERSION CHECK NOT COMPLETED");
+        showMessage(QLatin1String("DUMPER VERSION CHECK NOT COMPLETED"));
     }
 }
 
@@ -1356,9 +1395,9 @@ void GdbEngine::handleVarListChildrenHelperClassic(const GdbMi &item,
         data.setChildrenUnneeded();
         QByteArray cmd = "-var-list-children --all-values \"" + data.variable + '"';
         //iname += '.' + exp;
-        postCommand(cmd, WatchUpdate,
+        postCommand(cmd, Discardable,
             CB(handleVarListChildrenClassic), QVariant::fromValue(data));
-    } else if (!startsWithDigit(exp)
+    } else if (!startsWithDigit(QLatin1String(exp))
             && item.findChild("numchild").data() == "0") {
         // Happens for structs without data, e.g. interfaces.
         WatchData data;
@@ -1375,7 +1414,7 @@ void GdbEngine::handleVarListChildrenHelperClassic(const GdbMi &item,
         WatchData data;
         data.iname = name;
         QByteArray cmd = "-var-list-children --all-values \"" + data.variable + '"';
-        postCommand(cmd, WatchUpdate,
+        postCommand(cmd, Discardable,
             CB(handleVarListChildrenClassic), QVariant::fromValue(data));
     } else if (exp == "staticMetaObject") {
         //    && item.findChild("type").data() == "const QMetaObject")
@@ -1397,7 +1436,7 @@ void GdbEngine::handleVarListChildrenHelperClassic(const GdbMi &item,
             data.setChildrenUnneeded();
 
         data.name = _(exp);
-        if (data.type == data.name) {
+        if (data.name == QLatin1String(data.type)) {
             if (isPointerType(parent.type)) {
                 data.exp = "*(" + parent.exp + ')';
                 data.name = _("*") + parent.name;
@@ -1410,9 +1449,9 @@ void GdbEngine::handleVarListChildrenHelperClassic(const GdbMi &item,
             data.exp = "*(" + parent.exp + ')';
         } else if (startsWithDigit(data.name)) {
             // An array. No variables needed?
-            data.name = _c('[') + data.name + _c(']');
+            data.name = QLatin1Char('[') + data.name + QLatin1Char(']');
             data.exp = parent.exp + '[' + exp + ']';
-        } else if (0 && parent.name.endsWith(_c('.'))) {
+        } else if (0 && parent.name.endsWith(QLatin1Char('.'))) {
             // Happens with anonymous unions
             data.exp = parent.exp + data.name.toLatin1();
             //data.name = "<anonymous union>";
@@ -1449,9 +1488,6 @@ void GdbEngine::handleVarListChildrenClassic(const GdbResponse &response)
         //qDebug() <<  "VAR_LIST_CHILDREN: PARENT" << data.toString();
         QList<GdbMi> children = response.data.findChild("children").children();
 
-        for (int i = 0; i != children.size(); ++i)
-            handleVarListChildrenHelperClassic(children.at(i), data, i);
-
         if (children.isEmpty()) {
             // happens e.g. if no debug information is present or
             // if the class really has no children
@@ -1464,14 +1500,18 @@ void GdbEngine::handleVarListChildrenClassic(const GdbResponse &response)
             insertData(data1);
             data.setAllUnneeded();
             insertData(data);
-        } else if (data.variable.endsWith("private")
+        } else {
+            if (data.variable.endsWith("private")
                 || data.variable.endsWith("protected")
                 || data.variable.endsWith("public")) {
             // this skips the spurious "public", "private" etc levels
             // gdb produces
-        } else {
-            data.setChildrenUnneeded();
-            insertData(data);
+            } else {
+                data.setChildrenUnneeded();
+                insertData(data);
+            }
+            for (int i = 0; i != children.size(); ++i)
+                handleVarListChildrenHelperClassic(children.at(i), data, i);
         }
     } else {
         data.setError(QString::fromLocal8Bit(response.data.findChild("msg").data()));
