@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef DEBUGGER_GDBENGINE_H
 #define DEBUGGER_GDBENGINE_H
@@ -36,27 +33,29 @@
 #include "debuggerengine.h"
 
 #include "stackframe.h"
+#include "watchhandler.h"
 #include "watchutils.h"
 
-#include <QtCore/QByteArray>
-#include <QtCore/QProcess>
-#include <QtCore/QHash>
-#include <QtCore/QMap>
-#include <QtCore/QMultiMap>
-#include <QtCore/QObject>
-#include <QtCore/QPoint>
-#include <QtCore/QSet>
-#include <QtCore/QTextCodec>
-#include <QtCore/QTime>
-#include <QtCore/QTimer>
-#include <QtCore/QVariant>
+#include <QByteArray>
+#include <QProcess>
+#include <QHash>
+#include <QMap>
+#include <QMultiMap>
+#include <QObject>
+#include <QPoint>
+#include <QSet>
+#include <QTextCodec>
+#include <QTime>
+#include <QTimer>
+#include <QVariant>
 
 
 namespace Debugger {
 namespace Internal {
 
-class AbstractGdbAdapter;
 class AbstractGdbProcess;
+class DebugInfoTask;
+class DebugInfoTaskHandler;
 class GdbResponse;
 class GdbMi;
 class GdbToolTipContext;
@@ -65,11 +64,6 @@ class WatchData;
 class DisassemblerAgentCookie;
 class DisassemblerLines;
 
-class AttachGdbAdapter;
-class CoreGdbAdapter;
-class LocalPlainGdbAdapter;
-class RemoteGdbServerAdapter;
-
 enum DebuggingHelperState
 {
     DebuggingHelperUninitialized,
@@ -78,22 +72,6 @@ enum DebuggingHelperState
     DebuggingHelperUnavailable
 };
 
-
-enum GdbTestCase
-{
-    GdbTestNoBoundsOfCurrentFunction = 1
-};
-
-class UpdateParameters
-{
-public:
-    UpdateParameters() { tryPartial = tooltipOnly = false; }
-
-    bool tryPartial;
-    bool tooltipOnly;
-    QByteArray varList;
-};
-
 /* This is only used with Mac gdb since 2.2
  *
  * "Custom dumper" is a library compiled against the current
@@ -167,9 +145,10 @@ public:
     QString toString(bool debug = false) const;
 
     static QString msgDumperOutdated(double requiredVersion, double currentVersion);
+    static QString msgPtraceError(DebuggerStartMode sm);
 
 private:
-    typedef QMap<QString, Type> NameTypeMap;
+    typedef QMap<QByteArray, Type> NameTypeMap;
     typedef QMap<QByteArray, int> SizeCache;
 
     // Look up a simple (namespace) type
@@ -215,37 +194,24 @@ class GdbEngine : public Debugger::DebuggerEngine
     Q_OBJECT
 
 public:
-    GdbEngine(const DebuggerStartParameters &startParameters,
-        DebuggerEngine *masterEngine);
+    explicit GdbEngine(const DebuggerStartParameters &startParameters);
     ~GdbEngine();
-    AbstractGdbAdapter *gdbAdapter() const { return m_gdbAdapter; }
-
-private:
-    friend class AbstractGdbAdapter;
-    friend class AbstractPlainGdbAdapter;
-    friend class AttachGdbAdapter;
-    friend class CoreGdbAdapter;
-    friend class LocalPlainGdbAdapter;
-    friend class TermGdbAdapter;
-    friend class RemoteGdbServerAdapter;
-    friend class RemotePlainGdbAdapter;
-    friend class CodaGdbAdapter;
 
 private: ////////// General Interface //////////
 
-    virtual void setupEngine();
-    virtual void setupInferior();
-    virtual void runEngine();
-
-    virtual unsigned debuggerCapabilities() const;
-    virtual void detachDebugger();
-    virtual void shutdownEngine();
-    virtual void shutdownInferior();
+    virtual void setupEngine() = 0;
+    virtual void handleGdbStartFailed();
+    virtual void setupInferior() = 0;
     virtual void notifyInferiorSetupFailed();
+
+    virtual bool hasCapability(unsigned) const;
+    virtual void detachDebugger();
+    virtual void shutdownInferior();
+    virtual void shutdownEngine() = 0;
     virtual void abortDebugger();
 
     virtual bool acceptsDebuggerCommands() const;
-    virtual void executeDebuggerCommand(const QString &command);
+    virtual void executeDebuggerCommand(const QString &command, DebuggerLanguages languages);
     virtual QByteArray qtNamespace() const { return m_dumperHelper.qtNamespace(); }
     virtual void setQtNamespace(const QByteArray &ns)
         { return m_dumperHelper.setQtNamespace(ns); }
@@ -257,19 +223,15 @@ private: ////////// General State //////////
 
     bool m_registerNamesListed;
 
-private: ////////// Gdb Process Management //////////
+protected: ////////// Gdb Process Management //////////
 
-    AbstractGdbAdapter *createAdapter();
-    bool startGdb(const QStringList &args = QStringList(),
-                  const QString &settingsIdHint = QString());
+    void startGdb(const QStringList &args = QStringList());
+    void reportEngineSetupOk(const GdbResponse &response);
     void handleInferiorShutdown(const GdbResponse &response);
     void handleGdbExit(const GdbResponse &response);
-    void handleRemoteSetupDone(int gdbServerPort, int qmlPort);
-    void handleRemoteSetupFailed(const QString &message);
     void handleNamespaceExtraction(const GdbResponse &response);
 
-    void handleAdapterStarted();
-    void defaultInferiorShutdown(const char *cmd);
+    void loadInitScript();
     void loadPythonDumpers();
     void pythonDumpersFailed();
 
@@ -284,6 +246,8 @@ private: ////////// Gdb Process Management //////////
     // This notifies the base of a successful inferior setup.
     void finishInferiorSetup();
 
+    void handleDebugInfoLocation(const GdbResponse &response);
+
     // The adapter is still running just fine, but it failed to acquire a debuggee.
     void notifyInferiorSetupFailed(const QString &msg);
 
@@ -308,8 +272,6 @@ private:
     QByteArray m_inbuffer;
     bool m_busy;
 
-    AbstractGdbAdapter *m_gdbAdapter;
-
     // Name of the convenience variable containing the last
     // known function return value.
     QByteArray m_resultVarName;
@@ -323,9 +285,6 @@ private: ////////// Gdb Command Management //////////
         NeedsStop = 1,
         // No need to wait for the reply before continuing inferior.
         Discardable = 2,
-        // Trigger watch model rebuild when no such commands are pending anymore.
-        RebuildWatchModel = 4,
-        WatchUpdate = Discardable | RebuildWatchModel,
         // We can live without receiving an answer.
         NonCriticalResponse = 8,
         // Callback expects GdbResultRunning instead of GdbResultDone.
@@ -342,22 +301,18 @@ private: ////////// Gdb Command Management //////////
         ConsoleCommand = 512
     };
     Q_DECLARE_FLAGS(GdbCommandFlags, GdbCommandFlag)
-    private:
 
-    typedef void (GdbEngine::*GdbCommandCallback)
-        (const GdbResponse &response);
-    typedef void (AbstractGdbAdapter::*AdapterCallback)
-        (const GdbResponse &response);
+    protected:
+    typedef void (GdbEngine::*GdbCommandCallback)(const GdbResponse &response);
 
     struct GdbCommand
     {
         GdbCommand()
-            : flags(0), callback(0), adapterCallback(0), callbackName(0)
+            : flags(0), callback(0), callbackName(0)
         {}
 
         int flags;
         GdbCommandCallback callback;
-        AdapterCallback adapterCallback;
         const char *callbackName;
         QByteArray command;
         QVariant cookie;
@@ -369,6 +324,7 @@ private: ////////// Gdb Command Management //////////
     // send and decrements on receipt, effectively preventing
     // watch model updates before everything is finished.
     void flushCommand(const GdbCommand &cmd);
+protected:
     void postCommand(const QByteArray &command,
                      GdbCommandFlags flags,
                      GdbCommandCallback callback = 0,
@@ -378,15 +334,7 @@ private: ////////// Gdb Command Management //////////
                      GdbCommandCallback callback = 0,
                      const char *callbackName = 0,
                      const QVariant &cookie = QVariant());
-    void postCommand(const QByteArray &command,
-                     AdapterCallback callback,
-                     const char *callbackName,
-                     const QVariant &cookie = QVariant());
-    void postCommand(const QByteArray &command,
-                     GdbCommandFlags flags,
-                     AdapterCallback callback,
-                     const char *callbackName,
-                     const QVariant &cookie = QVariant());
+private:
     void postCommandHelper(const GdbCommand &cmd);
     void flushQueuedCommands();
     Q_SLOT void commandTimeout();
@@ -406,8 +354,8 @@ private: ////////// Gdb Command Management //////////
     // of evaluation. Responses with older tokens are considers
     // out of date and discarded.
     int m_oldestAcceptableToken;
+    int m_nonDiscardableCount;
 
-    int m_pendingWatchRequests; // Watch updating commands in flight
     int m_pendingBreakpointRequests; // Watch updating commands in flight
 
     typedef void (GdbEngine::*CommandsDoneCallback)();
@@ -416,15 +364,17 @@ private: ////////// Gdb Command Management //////////
 
     QList<GdbCommand> m_commandsToRunOnTemporaryBreak;
     int gdbVersion() const { return m_gdbVersion; }
+    void checkForReleaseBuild();
 
 private: ////////// Gdb Output, State & Capability Handling //////////
-
+protected:
     Q_SLOT void handleResponse(const QByteArray &buff);
     void handleStopResponse(const GdbMi &data);
     void handleResultRecord(GdbResponse *response);
-    void handleStop0(const GdbMi &data);
     void handleStop1(const GdbResponse &response);
     void handleStop1(const GdbMi &data);
+    void handleStop2(const GdbResponse &response);
+    void handleStop2(const GdbMi &data);
     Q_SLOT void handleStop2();
     StackFrame parseStackFrame(const GdbMi &mi, int level);
     void resetCommandQueue();
@@ -441,6 +391,7 @@ private: ////////// Gdb Output, State & Capability Handling //////////
     int m_gdbVersion; // 6.8.0 is 60800
     int m_gdbBuildVersion; // MAC only?
     bool m_isMacGdb;
+    bool m_isQnxGdb;
     bool m_hasBreakpointNotifications;
     bool m_hasPython;
     bool m_hasInferiorThreadList;
@@ -460,11 +411,12 @@ private: ////////// Inferior Management //////////
     void executeStepI();
     void executeNextI();
 
+    protected:
     void continueInferiorInternal();
-    void doNotifyInferiorRunOk();
     void autoContinueInferior();
     void continueInferior();
     void interruptInferior();
+    virtual void interruptInferior2() {}
     void interruptInferiorTemporarily();
 
     void executeRunToLine(const ContextData &data);
@@ -482,12 +434,12 @@ private: ////////// Inferior Management //////////
     void maybeHandleInferiorPidChanged(const QString &pid);
     void handleInfoProc(const GdbResponse &response);
 
-    QByteArray m_entryPoint;
-
 private: ////////// View & Data Stuff //////////
+    protected:
 
     void selectThread(int index);
     void activateFrame(int index);
+    void resetLocation();
 
     //
     // Breakpoint specific stuff
@@ -504,6 +456,7 @@ private: ////////// View & Data Stuff //////////
     void handleTraceInsert2(const GdbResponse &response);
     void handleBreakCondition(const GdbResponse &response);
     void handleBreakThreadSpec(const GdbResponse &response);
+    void handleBreakLineNumber(const GdbResponse &response);
     void handleWatchInsert(const GdbResponse &response);
     void handleCatchInsert(const GdbResponse &response);
     void handleInfoLine(const GdbResponse &response);
@@ -519,7 +472,7 @@ private: ////////// View & Data Stuff //////////
     // Modules specific stuff
     //
     void loadSymbols(const QString &moduleName);
-    void loadAllSymbols();
+    Q_SLOT void loadAllSymbols();
     void loadSymbolsForStack();
     void requestModuleSymbols(const QString &moduleName);
     void reloadModules();
@@ -528,8 +481,6 @@ private: ////////// View & Data Stuff //////////
     void handleModulesList(const GdbResponse &response);
     void handleShowModuleSymbols(const GdbResponse &response);
 
-    bool m_modulesListOutdated;
-
     //
     // Snapshot specific stuff
     //
@@ -543,6 +494,7 @@ private: ////////// View & Data Stuff //////////
     void setRegisterValue(int nr, const QString &value);
     void handleRegisterListNames(const GdbResponse &response);
     void handleRegisterListValues(const GdbResponse &response);
+    QVector<int> m_registerNumbers; // Map GDB register numbers to indices
 
     //
     // Disassembler specific stuff
@@ -587,15 +539,16 @@ private: ////////// View & Data Stuff //////////
     // awful hack to keep track of used files
     QMap<QString, QString> m_shortToFullName;
     QMap<QString, QString> m_fullToShortName;
+    QMultiMap<QString, QString> m_baseNameToFullName;
 
     void invalidateSourcesList();
-    bool m_sourcesListOutdated;
     bool m_sourcesListUpdating;
     bool m_breakListOutdated;
 
     //
     // Stack specific stuff
     //
+protected:
     void updateAll();
         void updateAllClassic();
         void updateAllPython();
@@ -629,13 +582,11 @@ private: ////////// View & Data Stuff //////////
     virtual void watchPoint(const QPoint &);
     void handleWatchPoint(const GdbResponse &response);
 
-    // FIXME: BaseClass. called to improve situation for a watch item
     void updateSubItemClassic(const WatchData &data);
 
-    void virtual updateWatchData(const WatchData &data, const WatchUpdateFlags &flags);
-    Q_SLOT void updateWatchDataHelper(const WatchData &data);
+    void updateWatchData(const WatchData &data, const WatchUpdateFlags &flags);
     void rebuildWatchModel();
-    bool showToolTip();
+    void showToolTip();
 
     void insertData(const WatchData &data);
     void sendWatchParameters(const QByteArray &params0);
@@ -659,6 +610,9 @@ private: ////////// View & Data Stuff //////////
     void handleDebuggingHelperVersionCheckClassic(const GdbResponse &response);
     void handleDetach(const GdbResponse &response);
 
+    void handleThreadGroupCreated(const GdbMi &result);
+    void handleThreadGroupExited(const GdbMi &result);
+
     Q_SLOT void createFullBacktrace();
     void handleCreateFullBacktrace(const GdbResponse &response);
 
@@ -668,7 +622,6 @@ private: ////////// View & Data Stuff //////////
             void handleStackFramePython(const GdbResponse &response);
 
     void handleStackListLocalsClassic(const GdbResponse &response);
-    void handleStackListLocalsPython(const GdbResponse &response);
 
     WatchData localVariable(const GdbMi &item,
                             const QStringList &uninitializedVariables,
@@ -692,8 +645,6 @@ private: ////////// View & Data Stuff //////////
     bool checkDebuggingHelpersClassic();
     void setDebuggingHelperStateClassic(DebuggingHelperState);
     void tryLoadDebuggingHelpersClassic();
-    void tryQueryDebuggingHelpersClassic();
-    Q_SLOT void setUseDebuggingHelpers(const QVariant &on);
 
     DebuggingHelperState m_debuggingHelperState;
     DumperHelper m_dumperHelper;
@@ -708,7 +659,6 @@ private: ////////// View & Data Stuff //////////
 
     static QByteArray tooltipIName(const QString &exp);
     QString tooltipExpression() const;
-    void clearToolTip();
     QScopedPointer<GdbToolTipContext> m_toolTipContext;
 
 
@@ -733,13 +683,57 @@ private: ////////// View & Data Stuff //////////
     // HACK:
     QByteArray m_currentThread;
     QString m_lastWinException;
+    QString m_lastMissingDebugInfo;
     BreakpointResponseId m_qFatalBreakpointResponseId;
-    bool m_actingOnExpectedStop;
+    bool m_terminalTrap;
+
+    bool usesExecInterrupt() const;
 
     QHash<int, QByteArray> m_scheduledTestResponses;
     QSet<int> m_testCases;
+
+    // Debug information
+    friend class DebugInfoTaskHandler;
+    void requestDebugInformation(const DebugInfoTask &task);
+    DebugInfoTaskHandler *m_debugInfoTaskHandler;
+
+    // Indicates whether we had at least one full attempt to load
+    // debug information.
+    bool attemptQuickStart() const;
+    bool m_fullStartDone;
+
+    // Test
+    bool m_forceAsyncModel;
+    QList<WatchData> m_completed;
+    QSet<QByteArray> m_uncompleted;
+
+    static QString msgGdbStopFailed(const QString &why);
+    static QString msgInferiorStopFailed(const QString &why);
+    static QString msgAttachedToStoppedInferior();
+    static QString msgInferiorSetupOk();
+    static QString msgInferiorRunOk();
+    static QString msgConnectRemoteServerFailed(const QString &why);
+
+protected:
+    enum DumperHandling
+    {
+        DumperNotAvailable,
+        DumperLoadedByAdapter,
+        DumperLoadedByGdbPreload,
+        DumperLoadedByGdb
+    };
+
+    virtual void write(const QByteArray &data);
+
+    virtual AbstractGdbProcess *gdbProc() = 0;
+    virtual DumperHandling dumperHandling() const = 0;
+
+protected:
+    bool prepareCommand();
+    void interruptLocalInferior(qint64 pid);
 };
 
+
 } // namespace Internal
 } // namespace Debugger
 
