@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "fakevimplugin.h"
 
@@ -44,9 +41,9 @@
 #include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/openeditorsmodel.h>
-#include <coreplugin/filemanager.h>
+#include <coreplugin/documentmanager.h>
 #include <coreplugin/icore.h>
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/id.h>
 #include <coreplugin/statusbarwidget.h>
@@ -78,51 +75,39 @@
 #include <utils/treewidgetcolumnstretcher.h>
 #include <utils/stylehelper.h>
 
-#include <cppeditor/cppeditorconstants.h>
-
 #include <cpptools/cpptoolsconstants.h>
 
-#include <QtCore/QAbstractTableModel>
-#include <QtCore/QDebug>
-#include <QtCore/QFile>
-#include <QtCore/QtPlugin>
-#include <QtCore/QObject>
-#include <QtCore/QSettings>
-#include <QtCore/QTextStream>
+#include <QAbstractTableModel>
+#include <QDebug>
+#include <QFile>
+#include <QtPlugin>
+#include <QObject>
+#include <QSettings>
+#include <QTextStream>
 
-#include <QtGui/QDesktopServices>
-#include <QtGui/QItemDelegate>
-#include <QtGui/QMessageBox>
-#include <QtGui/QPlainTextEdit>
-#include <QtGui/QShortcut>
-#include <QtGui/QTextBlock>
-#include <QtGui/QTextCursor>
-#include <QtGui/QTextEdit>
-#include <QtGui/QTreeWidgetItem>
+#include <QDesktopServices>
+#include <QItemDelegate>
+#include <QMessageBox>
+#include <QPlainTextEdit>
+#include <QShortcut>
+#include <QTextBlock>
+#include <QTextCursor>
+#include <QTextEdit>
+#include <QTreeWidgetItem>
 
-using namespace FakeVim::Internal;
 using namespace TextEditor;
 using namespace Core;
 
 namespace FakeVim {
-namespace Constants {
+namespace Internal {
 
 const char INSTALL_HANDLER[]                = "TextEditor.FakeVimHandler";
-const char MINI_BUFFER[]                    = "TextEditor.FakeVimMiniBuffer";
-const char INSTALL_KEY[]                    = "Alt+V,Alt+V";
 const char SETTINGS_CATEGORY[]              = "D.FakeVim";
 const char SETTINGS_CATEGORY_FAKEVIM_ICON[] = ":/core/images/category_fakevim.png";
 const char SETTINGS_ID[]                    = "A.General";
 const char SETTINGS_EX_CMDS_ID[]            = "B.ExCommands";
 const char SETTINGS_USER_CMDS_ID[]          = "C.UserCommands";
 
-} // namespace Constants
-} // namespace FakeVim
-
-
-namespace FakeVim {
-namespace Internal {
-
 class MiniBuffer : public QLabel
 {
     Q_OBJECT
@@ -178,23 +163,22 @@ typedef QMap<QString, QRegExp> ExCommandMap;
 typedef QMap<int, QString> UserCommandMap;
 typedef QLatin1String _;
 
-class FakeVimOptionPage : public Core::IOptionsPage
+class FakeVimOptionPage : public IOptionsPage
 {
     Q_OBJECT
 
 public:
-    FakeVimOptionPage() {}
-
-    // IOptionsPage
-    QString id() const { return _(Constants::SETTINGS_ID); }
-    QString displayName() const { return tr("General"); }
-    QString category() const { return _(Constants::SETTINGS_CATEGORY); }
-    QString displayCategory() const { return tr("FakeVim"); }
-    QIcon categoryIcon() const
-        { return QIcon(_(Constants::SETTINGS_CATEGORY_FAKEVIM_ICON)); }
+    FakeVimOptionPage()
+    {
+        setId(_(SETTINGS_ID));
+        setDisplayName(tr("General"));
+        setCategory(_(SETTINGS_CATEGORY));
+        setDisplayCategory(tr("FakeVim"));
+        setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
+    }
 
     QWidget *createPage(QWidget *parent);
-    void apply() { m_group.apply(ICore::instance()->settings()); }
+    void apply() { m_group.apply(ICore::settings()); }
     void finish() { m_group.finish(); }
     virtual bool matches(const QString &) const;
 
@@ -251,6 +235,8 @@ QWidget *FakeVimOptionPage::createPage(QWidget *parent)
         m_ui.checkBoxIncSearch);
     m_group.insert(theFakeVimSetting(ConfigUseCoreSearch),
         m_ui.checkBoxUseCoreSearch);
+    m_group.insert(theFakeVimSetting(ConfigSmartCase),
+        m_ui.checkBoxSmartCase);
 
     connect(m_ui.pushButtonCopyTextEditorSettings, SIGNAL(clicked()),
         SLOT(copyTextEditorSettings()));
@@ -272,6 +258,7 @@ QWidget *FakeVimOptionPage::createPage(QWidget *parent)
                 << sep << m_ui.checkBoxIncSearch->text()
                 << sep << m_ui.checkBoxStartOfLine->text()
                 << sep << m_ui.checkBoxUseCoreSearch->text()
+                << sep << m_ui.checkBoxSmartCase->text()
                 << sep << m_ui.checkBoxShowMarks->text()
                 << sep << m_ui.checkBoxPassControlKey->text()
                 << sep << m_ui.labelShiftWidth->text()
@@ -336,20 +323,20 @@ bool FakeVimOptionPage::matches(const QString &s) const
 
 enum { CommandRole = Qt::UserRole };
 
-class FakeVimExCommandsPage : public Core::CommandMappings
+class FakeVimExCommandsPage : public CommandMappings
 {
     Q_OBJECT
 
 public:
-    FakeVimExCommandsPage(FakeVimPluginPrivate *q) : m_q(q) {}
-    ~FakeVimExCommandsPage() {}
-
-    // IOptionsPage
-    QString id() const { return _(Constants::SETTINGS_EX_CMDS_ID); }
-    QString displayName() const { return tr("Ex Command Mapping"); }
-    QString category() const { return _(Constants::SETTINGS_CATEGORY); }
-    QString displayCategory() const { return tr("FakeVim"); }
-    QIcon categoryIcon() const { return QIcon(); } // TODO: Icon for FakeVim
+    FakeVimExCommandsPage(FakeVimPluginPrivate *q)
+        : m_q(q)
+    {
+        setId(_(SETTINGS_EX_CMDS_ID));
+        setDisplayName(tr("Ex Command Mapping"));
+        setCategory(_(SETTINGS_CATEGORY));
+        setDisplayCategory(tr("FakeVim"));
+        setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
+    }
 
     QWidget *createPage(QWidget *parent);
     void initialize();
@@ -380,17 +367,14 @@ QWidget *FakeVimExCommandsPage::createPage(QWidget *parent)
 
 void FakeVimExCommandsPage::initialize()
 {
-    ActionManager *am = ICore::instance()->actionManager();
-    QTC_ASSERT(am, return);
-
     QMap<QString, QTreeWidgetItem *> sections;
 
-    foreach (Command *c, am->commands()) {
+    foreach (Command *c, ActionManager::commands()) {
         if (c->action() && c->action()->isSeparator())
             continue;
 
         QTreeWidgetItem *item = new QTreeWidgetItem;
-        const QString name = Id::fromUniqueIdentifier(c->id()).toString();
+        const QString name = c->id().toString();
         const int pos = name.indexOf(QLatin1Char('.'));
         const QString section = name.left(pos);
         const QString subId = name.mid(pos + 1);
@@ -408,16 +392,7 @@ void FakeVimExCommandsPage::initialize()
         sections[section]->addChild(item);
 
         item->setText(0, subId);
-
-        if (c->action()) {
-            QString text = c->hasAttribute(Command::CA_UpdateText)
-                    && !c->defaultText().isNull()
-                ? c->defaultText() : c->action()->text();
-            text.remove(QRegExp("&(?!&)"));
-            item->setText(1, text);
-        } else {
-            item->setText(1, c->shortcut()->whatsThis());
-        }
+        item->setText(1, c->description());
 
         QString regex;
         if (exCommandMap().contains(name))
@@ -570,20 +545,21 @@ public:
     }
 };
 
-class FakeVimUserCommandsPage : public Core::IOptionsPage
+class FakeVimUserCommandsPage : public IOptionsPage
 {
     Q_OBJECT
 
 public:
-    FakeVimUserCommandsPage(FakeVimPluginPrivate *q) : m_q(q) {}
-    ~FakeVimUserCommandsPage() {}
+    FakeVimUserCommandsPage(FakeVimPluginPrivate *q)
+        : m_q(q)
+    {
+        setId(_(SETTINGS_USER_CMDS_ID));
+        setDisplayName(tr("User Command Mapping"));
+        setCategory(_(SETTINGS_CATEGORY));
+        setDisplayCategory(tr("FakeVim"));
+        setCategoryIcon(_(SETTINGS_CATEGORY_FAKEVIM_ICON));
+    }
 
-    // IOptionsPage
-    QString id() const { return _(Constants::SETTINGS_USER_CMDS_ID); }
-    QString displayName() const { return tr("User Command Mapping"); }
-    QString category() const { return _(Constants::SETTINGS_CATEGORY); }
-    QString displayCategory() const { return tr("FakeVim"); }
-    QIcon categoryIcon() const { return QIcon(); }
     void apply();
     void finish() {}
 
@@ -627,15 +603,15 @@ void FakeVimUserCommandsPage::apply()
 //
 ///////////////////////////////////////////////////////////////////////
 
-class FakeVimCompletionAssistProvider : public TextEditor::CompletionAssistProvider
+class FakeVimCompletionAssistProvider : public CompletionAssistProvider
 {
 public:
-    virtual bool supportsEditor(const QString &) const
+    bool supportsEditor(const Id &) const
     {
         return false;
     }
 
-    virtual TextEditor::IAssistProcessor *createProcessor() const;
+    IAssistProcessor *createProcessor() const;
 
     void setActive(const QString &needle, bool forward, FakeVimHandler *handler)
     {
@@ -725,17 +701,17 @@ public:
 class FakeVimCompletionAssistProcessor : public IAssistProcessor
 {
 public:
-    FakeVimCompletionAssistProcessor(const TextEditor::IAssistProvider *provider)
+    FakeVimCompletionAssistProcessor(const IAssistProvider *provider)
         : m_provider(static_cast<const FakeVimCompletionAssistProvider *>(provider))
     {}
 
-    virtual TextEditor::IAssistProposal *perform(const IAssistInterface *interface)
+    IAssistProposal *perform(const IAssistInterface *interface)
     {
         const QString &needle = m_provider->needle();
 
         const int basePosition = interface->position() - needle.size();
 
-        QTextCursor tc(interface->document());
+        QTextCursor tc(interface->textDocument());
         tc.setPosition(interface->position());
         tc.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
 
@@ -842,10 +818,10 @@ private:
     FakeVimOptionPage *m_fakeVimOptionsPage;
     FakeVimExCommandsPage *m_fakeVimExCommandsPage;
     FakeVimUserCommandsPage *m_fakeVimUserCommandsPage;
-    QHash<Core::IEditor *, FakeVimHandler *> m_editorToHandler;
+    QHash<IEditor *, FakeVimHandler *> m_editorToHandler;
 
-    void triggerAction(const QString &code);
-    void setActionChecked(const QString &code, bool check);
+    void triggerAction(const Id &id);
+    void setActionChecked(const Id &id, bool check);
 
     typedef int (*DistFunction)(const QRect &cursor, const QRect &other);
     void moveSomewhere(DistFunction f);
@@ -860,7 +836,7 @@ private:
     UserCommandMap m_userCommandMap;
     UserCommandMap m_defaultUserCommandMap;
 
-    Core::StatusBarWidget *m_statusBar;
+    StatusBarWidget *m_statusBar;
     // @TODO: Delete
     //WordCompletion *m_wordCompletion;
     FakeVimCompletionAssistProvider *m_wordProvider;
@@ -904,7 +880,7 @@ FakeVimPluginPrivate::FakeVimPluginPrivate(FakeVimPlugin *plugin)
         QRegExp("^(cN(ext)?|cp(revious)?)!?( (.*))?$");
     defaultExCommandMap()["Coreplugin.OutputPane.nextitem"] =
         QRegExp("^cn(ext)?!?( (.*))?$");
-    defaultExCommandMap()[CppEditor::Constants::JUMP_TO_DEFINITION] =
+    defaultExCommandMap()[TextEditor::Constants::FOLLOW_SYMBOL_UNDER_CURSOR] =
         QRegExp("^tag?$");
     defaultExCommandMap()[Core::Constants::GO_BACK] =
         QRegExp("^pop?$");
@@ -938,7 +914,7 @@ FakeVimPluginPrivate::~FakeVimPluginPrivate()
 void FakeVimPluginPrivate::onCoreAboutToClose()
 {
     // Don't attach to editors anymore.
-    disconnect(ICore::instance()->editorManager(), SIGNAL(editorOpened(Core::IEditor*)),
+    disconnect(ICore::editorManager(), SIGNAL(editorOpened(Core::IEditor*)),
         this, SLOT(editorOpened(Core::IEditor*)));
 }
 
@@ -948,10 +924,7 @@ void FakeVimPluginPrivate::aboutToShutdown()
 
 bool FakeVimPluginPrivate::initialize()
 {
-    Core::ICore *core = Core::ICore::instance();
-    EditorManager *editorManager = core->editorManager();
-    ActionManager *actionManager = core->actionManager();
-    QTC_ASSERT(actionManager, return false);
+    EditorManager *editorManager = ICore::editorManager();
 
     //m_wordCompletion = new WordCompletion;
     //q->addAutoReleasedObject(m_wordCompletion);
@@ -980,27 +953,26 @@ bool FakeVimPluginPrivate::initialize()
 
     readSettings();
 
-    Core::Command *cmd = 0;
-    cmd = actionManager->registerAction(theFakeVimSetting(ConfigUseFakeVim),
-        Constants::INSTALL_HANDLER, globalcontext, true);
-    cmd->setDefaultKeySequence(QKeySequence(Constants::INSTALL_KEY));
+    Command *cmd = 0;
+    cmd = ActionManager::registerAction(theFakeVimSetting(ConfigUseFakeVim),
+        INSTALL_HANDLER, globalcontext, true);
+    cmd->setDefaultKeySequence(QKeySequence(UseMacShortcuts ? tr("Meta+V,Meta+V") : tr("Alt+V,Alt+V")));
 
     ActionContainer *advancedMenu =
-        actionManager->actionContainer(Core::Constants::M_EDIT_ADVANCED);
+        ActionManager::actionContainer(Core::Constants::M_EDIT_ADVANCED);
     advancedMenu->addAction(cmd, Core::Constants::G_EDIT_EDITOR);
 
     for (int i = 1; i < 10; ++i) {
         QAction *act = new QAction(this);
-        act->setText(QString("Execute User Action #%1").arg(i));
+        act->setText(tr("Execute User Action #%1").arg(i));
         act->setData(i);
         QString id = QString("FakeVim.UserAction%1").arg(i);
-        QString keys = QString("Alt+V,%1").arg(i);
-        cmd = actionManager->registerAction(act, id, globalcontext);
-        cmd->setDefaultKeySequence(QKeySequence(keys));
+        cmd = ActionManager::registerAction(act, Id(id), globalcontext);
+        cmd->setDefaultKeySequence(QKeySequence((UseMacShortcuts ? tr("Meta+V,%1") : tr("Alt+V,%1")).arg(i)));
         connect(act, SIGNAL(triggered()), SLOT(userActionTriggered()));
     }
 
-    connect(core, SIGNAL(coreAboutToClose()), this, SLOT(onCoreAboutToClose()));
+    connect(ICore::instance(), SIGNAL(coreAboutToClose()), this, SLOT(onCoreAboutToClose()));
 
     // EditorManager
     connect(editorManager, SIGNAL(editorAboutToClose(Core::IEditor*)),
@@ -1033,23 +1005,22 @@ void FakeVimPluginPrivate::userActionTriggered()
     if (!key)
         return;
     QString cmd = userCommandMap().value(key);
-    Core::ICore *core = Core::ICore::instance();
-    EditorManager *editorManager = core->editorManager();
-    IEditor *editor = editorManager->currentEditor();
+    IEditor *editor = EditorManager::currentEditor();
     FakeVimHandler *handler = m_editorToHandler[editor];
     if (handler)
         handler->handleInput(cmd);
 }
 
-static const char exCommandMapGroup[] = "FakeVimExCommand";
-static const char userCommandMapGroup[] = "FakeVimUserCommand";
-static const char reKey[] = "RegEx";
-static const char cmdKey[] = "Cmd";
-static const char idKey[] = "Command";
+
+const char exCommandMapGroup[] = "FakeVimExCommand";
+const char userCommandMapGroup[] = "FakeVimUserCommand";
+const char reKey[] = "RegEx";
+const char cmdKey[] = "Cmd";
+const char idKey[] = "Command";
 
 void FakeVimPluginPrivate::writeSettings()
 {
-    QSettings *settings = ICore::instance()->settings();
+    QSettings *settings = ICore::settings();
 
     theFakeVimSettings()->writeSettings(settings);
 
@@ -1097,7 +1068,7 @@ void FakeVimPluginPrivate::writeSettings()
 
 void FakeVimPluginPrivate::readSettings()
 {
-    QSettings *settings = ICore::instance()->settings();
+    QSettings *settings = ICore::settings();
 
     theFakeVimSettings()->readSettings(settings);
 
@@ -1143,27 +1114,21 @@ void FakeVimPluginPrivate::maybeReadVimRc()
 
 void FakeVimPluginPrivate::showSettingsDialog()
 {
-    ICore::instance()->showOptionsDialog(
-        _(Constants::SETTINGS_CATEGORY),
-        _(Constants::SETTINGS_ID));
+    ICore::showOptionsDialog(_(SETTINGS_CATEGORY), _(SETTINGS_ID));
 }
 
-void FakeVimPluginPrivate::triggerAction(const QString &code)
+void FakeVimPluginPrivate::triggerAction(const Id &id)
 {
-    Core::ActionManager *am = ICore::instance()->actionManager();
-    QTC_ASSERT(am, return);
-    Core::Command *cmd = am->command(code);
-    QTC_ASSERT(cmd, qDebug() << "UNKNOWN CODE: " << code; return);
+    Core::Command *cmd = ActionManager::command(id);
+    QTC_ASSERT(cmd, qDebug() << "UNKNOWN CODE: " << id.name(); return);
     QAction *action = cmd->action();
     QTC_ASSERT(action, return);
     action->trigger();
 }
 
-void FakeVimPluginPrivate::setActionChecked(const QString &code, bool check)
+void FakeVimPluginPrivate::setActionChecked(const Id &id, bool check)
 {
-    Core::ActionManager *am = ICore::instance()->actionManager();
-    QTC_ASSERT(am, return);
-    Core::Command *cmd = am->command(code);
+    Core::Command *cmd = ActionManager::command(id);
     QTC_ASSERT(cmd, return);
     QAction *action = cmd->action();
     QTC_ASSERT(action, return);
@@ -1269,11 +1234,9 @@ void FakeVimPluginPrivate::windowCommand(int key)
 
 void FakeVimPluginPrivate::moveSomewhere(DistFunction f)
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
-    IEditor *editor = editorManager->currentEditor();
+    IEditor *editor = EditorManager::currentEditor();
     QWidget *w = editor->widget();
-    QPlainTextEdit *pe =
-        qobject_cast<QPlainTextEdit *>(editor->widget());
+    QPlainTextEdit *pe = qobject_cast<QPlainTextEdit *>(w);
     QTC_ASSERT(pe, return);
     QRect rc = pe->cursorRect();
     QRect cursorRect(w->mapToGlobal(rc.topLeft()),
@@ -1283,8 +1246,7 @@ void FakeVimPluginPrivate::moveSomewhere(DistFunction f)
     IEditor *bestEditor = 0;
     int bestValue = 1 << 30;
 
-    QList<IEditor*> editors = editorManager->visibleEditors();
-    foreach (IEditor *editor, editors) {
+    foreach (IEditor *editor, EditorManager::instance()->visibleEditors()) {
         QWidget *w = editor->widget();
         QRect editorRect(w->mapToGlobal(w->geometry().topLeft()),
                 w->mapToGlobal(w->geometry().bottomRight()));
@@ -1302,7 +1264,7 @@ void FakeVimPluginPrivate::moveSomewhere(DistFunction f)
     // FIME: This is know to fail as the EditorManager will fall back to
     // the current editor's view. Needs additional public API there.
     if (bestEditor)
-        editorManager->activateEditor(bestEditor);
+        EditorManager::activateEditor(bestEditor);
 }
 
 void FakeVimPluginPrivate::find(bool reverse)
@@ -1345,7 +1307,7 @@ public:
     }
 };
 
-void FakeVimPluginPrivate::editorOpened(Core::IEditor *editor)
+void FakeVimPluginPrivate::editorOpened(IEditor *editor)
 {
     if (!editor)
         return;
@@ -1400,7 +1362,7 @@ void FakeVimPluginPrivate::editorOpened(Core::IEditor *editor)
     connect(ICore::instance(), SIGNAL(saveSettingsRequested()),
         SLOT(writeSettings()));
 
-    handler->setCurrentFileName(editor->file()->fileName());
+    handler->setCurrentFileName(editor->document()->fileName());
     handler->installEventFilter();
 
     // pop up the bar
@@ -1410,7 +1372,7 @@ void FakeVimPluginPrivate::editorOpened(Core::IEditor *editor)
     }
 }
 
-void FakeVimPluginPrivate::editorAboutToClose(Core::IEditor *editor)
+void FakeVimPluginPrivate::editorAboutToClose(IEditor *editor)
 {
     //qDebug() << "CLOSING: " << editor << editor->widget();
     m_editorToHandler.remove(editor);
@@ -1429,18 +1391,18 @@ void FakeVimPluginPrivate::setUseFakeVimInternal(bool on)
 {
     if (on) {
         //ICore *core = ICore::instance();
-        //core->updateAdditionalContexts(Core::Context(FAKEVIM_CONTEXT),
-        // Core::Context());
-        foreach (Core::IEditor *editor, m_editorToHandler.keys())
+        //core->updateAdditionalContexts(Context(FAKEVIM_CONTEXT),
+        // Context());
+        foreach (IEditor *editor, m_editorToHandler.keys())
             m_editorToHandler[editor]->setupWidget();
     } else {
         //ICore *core = ICore::instance();
-        //core->updateAdditionalContexts(Core::Context(),
-        // Core::Context(FAKEVIM_CONTEXT));
+        //core->updateAdditionalContexts(Context(),
+        // Context(FAKEVIM_CONTEXT));
         showCommandBuffer(QString(), -1);
-        foreach (Core::IEditor *editor, m_editorToHandler.keys()) {
-            if (TextEditor::BaseTextEditorWidget *textEditor =
-                    qobject_cast<TextEditor::BaseTextEditorWidget *>(editor->widget())) {
+        foreach (IEditor *editor, m_editorToHandler.keys()) {
+            if (BaseTextEditorWidget *textEditor =
+                    qobject_cast<BaseTextEditorWidget *>(editor->widget())) {
                 m_editorToHandler[editor]->restoreWidget(textEditor->tabSettings().m_tabSize);
             }
         }
@@ -1505,11 +1467,11 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
     *handled = true;
     if (cmd.matches("w", "write") || cmd.cmd == "wq") {
         // :w[rite]
-        Core::IEditor *editor = m_editorToHandler.key(handler);
+        IEditor *editor = m_editorToHandler.key(handler);
         const QString fileName = handler->currentFileName();
-        if (editor && editor->file()->fileName() == fileName) {
+        if (editor && editor->document()->fileName() == fileName) {
             // Handle that as a special case for nicer interaction with core
-            Core::ICore::instance()->fileManager()->saveFile(editor->file());
+            DocumentManager::saveDocument(editor->document());
             // Check result by reading back.
             QFile file3(fileName);
             file3.open(QIODevice::ReadOnly);
@@ -1524,9 +1486,8 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         }
     } else if (cmd.matches("wa", "wall")) {
         // :w[all]
-        FileManager *fm = ICore::instance()->fileManager();
-        QList<IFile *> toSave = fm->modifiedFiles();
-        QList<IFile *> failed = fm->saveModifiedFilesSilently(toSave);
+        QList<IDocument *> toSave = DocumentManager::modifiedDocuments();
+        QList<IDocument *> failed = DocumentManager::saveModifiedDocumentsSilently(toSave);
         if (failed.isEmpty())
             handler->showBlackMessage(tr("Saving succeeded"));
         else
@@ -1575,15 +1536,21 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
         // :on[ly]
         //triggerAction(Core::Constants::REMOVE_ALL_SPLITS);
         triggerAction(Core::Constants::REMOVE_CURRENT_SPLIT);
+    } else if (cmd.cmd == "AS") {
+        triggerAction(Core::Constants::SPLIT);
+        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
+    } else if (cmd.cmd == "AV") {
+        triggerAction(Core::Constants::SPLIT_SIDE_BY_SIDE);
+        triggerAction(CppTools::Constants::SWITCH_HEADER_SOURCE);
     } else {
         // Check whether one of the configure commands matches.
         typedef ExCommandMap::const_iterator Iterator;
         const Iterator end = exCommandMap().constEnd();
         for (Iterator it = exCommandMap().constBegin(); it != end; ++it) {
             const QString &id = it.key();
-            const QRegExp &re = it.value();
+            QRegExp re = it.value();
             if (!re.pattern().isEmpty() && re.indexIn(cmd.cmd) != -1) {
-                triggerAction(id);
+                triggerAction(Core::Id(id));
                 return;
             }
         }
@@ -1591,15 +1558,15 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
     }
 }
 
-void FakeVimPluginPrivate::handleDelayedQuit(bool forced, Core::IEditor *editor)
+void FakeVimPluginPrivate::handleDelayedQuit(bool forced, IEditor *editor)
 {
     // This tries to simulate vim behaviour. But the models of vim and
     // Qt Creator core do not match well...
-    EditorManager *editorManager = ICore::instance()->editorManager();
+    EditorManager *editorManager = ICore::editorManager();
     if (editorManager->hasSplitter()) {
         triggerAction(Core::Constants::REMOVE_CURRENT_SPLIT);
     } else {
-        QList<Core::IEditor *> editors;
+        QList<IEditor *> editors;
         editors.append(editor);
         editorManager->closeEditors(editors, !forced);
     }
@@ -1608,7 +1575,7 @@ void FakeVimPluginPrivate::handleDelayedQuit(bool forced, Core::IEditor *editor)
 void FakeVimPluginPrivate::handleDelayedQuitAll(bool forced)
 {
     triggerAction(Core::Constants::REMOVE_ALL_SPLITS);
-    ICore::instance()->editorManager()->closeAllEditors(!forced);
+    ICore::editorManager()->closeAllEditors(!forced);
 }
 
 void FakeVimPluginPrivate::moveToMatchingParenthesis(bool *moved, bool *forward,
@@ -1714,16 +1681,15 @@ void FakeVimPluginPrivate::changeSelection
 
 int FakeVimPluginPrivate::currentFile() const
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
-    OpenEditorsModel *model = editorManager->openedEditorsModel();
-    IEditor *editor = editorManager->currentEditor();
+    OpenEditorsModel *model = EditorManager::instance()->openedEditorsModel();
+    IEditor *editor = EditorManager::currentEditor();
     return model->indexOf(editor).row();
 }
 
 void FakeVimPluginPrivate::switchToFile(int n)
 {
-    EditorManager *editorManager = ICore::instance()->editorManager();
-    Core::OpenEditorsModel *model = editorManager->openedEditorsModel();
+    EditorManager *editorManager = ICore::editorManager();
+    OpenEditorsModel *model = editorManager->openedEditorsModel();
     int size = model->rowCount();
     QTC_ASSERT(size, return);
     n = n % size;
@@ -1782,7 +1748,7 @@ ExtensionSystem::IPlugin::ShutdownFlag FakeVimPlugin::aboutToShutdown()
 
 void FakeVimPlugin::extensionsInitialized()
 {
-    d->m_statusBar = new Core::StatusBarWidget;
+    d->m_statusBar = new StatusBarWidget;
     d->m_statusBar->setWidget(new MiniBuffer);
     d->m_statusBar->setPosition(StatusBarWidget::Last);
     addAutoReleasedObject(d->m_statusBar);
@@ -1793,4 +1759,4 @@ void FakeVimPlugin::extensionsInitialized()
 
 #include "fakevimplugin.moc"
 
-Q_EXPORT_PLUGIN(FakeVimPlugin)
+Q_EXPORT_PLUGIN(FakeVim::Internal::FakeVimPlugin)
