@@ -1,20 +1,19 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** This file is part of Qt Creator.
 **
-** Contact: Nokia Corporation (qt-info@nokia.com)
-**
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
-**
 ** Alternatively, this file may be used under the terms of the GNU Lesser
 ** General Public License version 2.1 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.LGPL included in the
@@ -22,44 +21,50 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-**************************************************************************/
+****************************************************************************/
 
 #include "gitorious.h"
 
-#include <QtCore/QDebug>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QXmlStreamReader>
-#include <QtCore/QSettings>
+#include <QDebug>
+#include <QCoreApplication>
+#include <QXmlStreamReader>
+#include <QSettings>
 
-#include <QtNetwork/QNetworkAccessManager>
-#include <QtNetwork/QNetworkReply>
+#include <QNetworkReply>
+
+#include <utils/qtcassert.h>
+#include <utils/networkaccessmanager.h>
 
 enum { debug = 0 };
 
 enum Protocol { ListCategoriesProtocol, ListProjectsProtocol };
 
-static const char *protocolPropertyC = "gitoriousProtocol";
-static const char *hostNamePropertyC = "gitoriousHost";
-static const char *pagePropertyC = "requestPage";
+static const char protocolPropertyC[] = "gitoriousProtocol";
+static const char hostNamePropertyC[] = "gitoriousHost";
+static const char pagePropertyC[] = "requestPage";
 
-static const char *settingsKeyC = "GitoriousHosts";
+static const char settingsKeyC[] = "GitoriousHosts";
 
 // Gitorious paginates projects as 20 per page. It starts with page 1.
 enum { ProjectsPageSize = 20 };
 
-// Format an URL for a XML request
-static inline QUrl xmlRequest(const QString &host, const QString &request, int page = -1)
+// Format an URL for a http request
+static inline QUrl httpRequest(const QString &host, const QString &request)
 {
     QUrl url;
     url.setScheme(QLatin1String("http"));
-    url.setHost(host);
+    const QStringList hostList = host.split(QLatin1Char(':'), QString::SkipEmptyParts);
+    if (hostList.size() > 0)
+    {
+        url.setHost(hostList.at(0));
+        if (hostList.size() > 1)
+            url.setPort(hostList.at(1).toInt());
+    }
     url.setPath(QLatin1Char('/') + request);
-    url.addQueryItem(QLatin1String("format"), QLatin1String("xml"));
-    if (page >= 0)
-        url.addQueryItem(QLatin1String("page"), QString::number(page));
     return url;
 }
 
@@ -232,7 +237,7 @@ void GitoriousProjectReader::readProjects(QXmlStreamReader &reader)
             break;
 
         if (reader.isStartElement()) {
-            if (reader.name() == "project") {
+            if (reader.name() == QLatin1String("project")) {
                 const QSharedPointer<GitoriousProject> p = readProject(reader);
                 if (!p->name.isEmpty())
                     m_projects.push_back(p);
@@ -276,7 +281,7 @@ QList<GitoriousRepository> GitoriousProjectReader::readRepositories(QXmlStreamRe
     int defaultType = -1;
 
     // The "mainlines"/"clones" elements are not used in the
-    // Nokia setup, handle them optionally.
+    // QtProject setup, handle them optionally.
     while (!reader.atEnd()) {
         reader.readNext();
 
@@ -343,7 +348,7 @@ GitoriousRepository GitoriousProjectReader::readRepository(QXmlStreamReader &rea
 
 void GitoriousProjectReader::readUnknownElement(QXmlStreamReader &reader)
 {
-    Q_ASSERT(reader.isStartElement());
+    QTC_ASSERT(reader.isStartElement(), return);
 
     while (!reader.atEnd()) {
         reader.readNext();
@@ -442,7 +447,7 @@ void Gitorious::listCategoriesReply(int index, QByteArray dataB)
         const QString data = QString::fromUtf8(dataB);
         // Cut out the contents of the anchors
         QRegExp pattern = QRegExp(QLatin1String("<a href=[^>]+>([^<]+)</a>"));
-        Q_ASSERT(pattern.isValid());
+        QTC_CHECK(pattern.isValid());
         GitoriousHost::CategoryList &categories = m_hosts[index].categories;
         for (int pos = pattern.indexIn(data) ; pos != -1; ) {
             const QString cat = pattern.cap(1);
@@ -525,7 +530,7 @@ void Gitorious::slotReplyFinished()
 QNetworkReply *Gitorious::createRequest(const QUrl &url, int protocol, int hostIndex, int page)
 {
     if (!m_networkManager)
-        m_networkManager = new QNetworkAccessManager(this);
+        m_networkManager = new Utils::NetworkAccessManager(this);
     QNetworkReply *reply = m_networkManager->get(QNetworkRequest(url));
     connect(reply, SIGNAL(finished()), this, SLOT(slotReplyFinished()));
     reply->setProperty(protocolPropertyC, QVariant(protocol));
@@ -540,10 +545,7 @@ QNetworkReply *Gitorious::createRequest(const QUrl &url, int protocol, int hostI
 void Gitorious::updateCategories(int index)
 {
     // For now, parse the HTML of the projects site for "Popular Categories":
-    QUrl url;
-    url.setScheme(QLatin1String("http"));
-    url.setHost(hostName(index));
-    url.setPath(QLatin1String("/projects"));
+    const QUrl url = httpRequest(hostName(index), QLatin1String("projects"));
     createRequest(url, ListCategoriesProtocol, index);
 }
 
@@ -554,7 +556,10 @@ void Gitorious::updateProjectList(int hostIndex)
 
 void Gitorious::startProjectsRequest(int hostIndex, int page)
 {
-    const QUrl url = xmlRequest(hostName(hostIndex), QLatin1String("projects"), page);
+    QUrl url = httpRequest(hostName(hostIndex), QLatin1String("projects"));
+    url.addQueryItem(QLatin1String("format"), QLatin1String("xml"));
+    if (page >= 0)
+        url.addQueryItem(QLatin1String("page"), QString::number(page));
     createRequest(url, ListProjectsProtocol, hostIndex, page);
 }
 
