@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "glslcompletionassist.h"
 #include "glsleditorconstants.h"
@@ -43,7 +40,7 @@
 #include <glsl/glslsymbols.h>
 #include <glsl/glslastdump.h>
 
-#include <coreplugin/ifile.h>
+#include <coreplugin/idocument.h>
 #include <texteditor/completionsettings.h>
 #include <texteditor/codeassist/basicproposalitem.h>
 #include <texteditor/codeassist/basicproposalitemlistmodel.h>
@@ -52,14 +49,14 @@
 #include <cplusplus/ExpressionUnderCursor.h>
 #include <utils/faketooltip.h>
 
-#include <QtGui/QIcon>
-#include <QtGui/QPainter>
-#include <QtGui/QLabel>
-#include <QtGui/QToolButton>
-#include <QtGui/QHBoxLayout>
-#include <QtGui/QApplication>
-#include <QtGui/QDesktopWidget>
-#include <QtCore/QDebug>
+#include <QIcon>
+#include <QPainter>
+#include <QLabel>
+#include <QToolButton>
+#include <QHBoxLayout>
+#include <QApplication>
+#include <QDesktopWidget>
+#include <QDebug>
 
 using namespace GLSLEditor;
 using namespace Internal;
@@ -122,9 +119,9 @@ bool checkStartOfIdentifier(const QString &word)
 // ----------------------------
 // GLSLCompletionAssistProvider
 // ----------------------------
-bool GLSLCompletionAssistProvider::supportsEditor(const QString &editorId) const
+bool GLSLCompletionAssistProvider::supportsEditor(const Core::Id &editorId) const
 {
-    return editorId == QLatin1String(Constants::C_GLSLEDITOR_ID);
+    return editorId == Constants::C_GLSLEDITOR_ID;
 }
 
 IAssistProcessor *GLSLCompletionAssistProvider::createProcessor() const
@@ -239,7 +236,7 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
     bool functionCall = (ch == QLatin1Char('(') && pos == m_interface->position() - 1);
 
     if (ch == QLatin1Char(',')) {
-        QTextCursor tc(m_interface->document());
+        QTextCursor tc(m_interface->textDocument());
         tc.setPosition(pos);
         const int start = expressionUnderCursor.startOfFunctionCall(tc);
         if (start == -1)
@@ -254,7 +251,7 @@ IAssistProposal *GLSLCompletionAssistProcessor::perform(const IAssistInterface *
 
     if (ch == QLatin1Char('.') || functionCall) {
         const bool memberCompletion = ! functionCall;
-        QTextCursor tc(m_interface->document());
+        QTextCursor tc(m_interface->textDocument());
         tc.setPosition(pos);
 
         // get the expression under cursor
@@ -465,13 +462,13 @@ void GLSLCompletionAssistProcessor::addCompletion(const QString &text,
 // -----------------------------
 // GLSLCompletionAssistInterface
 // -----------------------------
-GLSLCompletionAssistInterface::GLSLCompletionAssistInterface(QTextDocument *document,
+GLSLCompletionAssistInterface::GLSLCompletionAssistInterface(QTextDocument *textDocument,
                                                              int position,
-                                                             Core::IFile *file,
+                                                             Core::IDocument *document,
                                                              TextEditor::AssistReason reason,
                                                              const QString &mimeType,
                                                              const Document::Ptr &glslDoc)
-    : DefaultAssistInterface(document, position, file, reason)
+    : DefaultAssistInterface(textDocument, position, document, reason)
     , m_mimeType(mimeType)
     , m_glslDoc(glslDoc)
 {
