@@ -1,52 +1,50 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #ifndef HELPVIEWER_H
 #define HELPVIEWER_H
 
 #include <find/ifindsupport.h>
 
-#include <QtCore/qglobal.h>
-#include <QtCore/QString>
-#include <QtCore/QUrl>
-#include <QtCore/QVariant>
+#include <qglobal.h>
+#include <QString>
+#include <QUrl>
+#include <QVariant>
 
-#include <QtGui/QAction>
-#include <QtGui/QFont>
+#include <QAction>
+#include <QFont>
 
 #if defined(QT_NO_WEBKIT)
-#include <QtGui/QTextBrowser>
+#include <QTextBrowser>
 #else
-#include <QtWebKit/QWebView>
+#include <QWebPage>
+#include <QWebView>
 #endif
 
 namespace Help {
@@ -100,6 +98,7 @@ public:
 public slots:
     void copy();
     void home();
+    void stop();
 
     void forward();
     void backward();
@@ -141,6 +140,33 @@ private:
     HelpViewerPrivate *d;
 };
 
+#ifndef QT_NO_WEBKIT
+class HelpPage : public QWebPage
+{
+    Q_OBJECT
+public:
+    HelpPage(QObject *parent);
+
+protected:
+    virtual QWebPage *createWindow(QWebPage::WebWindowType);
+    virtual void triggerAction(WebAction action, bool checked = false);
+
+    virtual bool acceptNavigationRequest(QWebFrame *frame,
+        const QNetworkRequest &request, NavigationType type);
+
+private slots:
+    void onHandleUnsupportedContent(QNetworkReply *reply);
+
+private:
+    QUrl m_loadingUrl;
+    bool closeNewTabIfNeeded;
+
+    friend class Help::Internal::HelpViewer;
+    Qt::MouseButtons m_pressedButtons;
+    Qt::KeyboardModifiers m_keyboardModifiers;
+};
+#endif // QT_NO_WEBKIT
+
 }   // namespace Internal
 }   // namespace Help
 
