@@ -1,34 +1,31 @@
 /**************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (c) 2013 Brian McGillion
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2009 Brian McGillion
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "optionspage.h"
 #include "mercurialsettings.h"
@@ -38,7 +35,7 @@
 #include <utils/pathchooser.h>
 #include <vcsbase/vcsbaseconstants.h>
 
-#include <QtCore/QTextStream>
+#include <QTextStream>
 
 using namespace Mercurial::Internal;
 using namespace Mercurial;
@@ -54,7 +51,7 @@ OptionsPageWidget::OptionsPageWidget(QWidget *parent) :
 MercurialSettings OptionsPageWidget::settings() const
 {
     MercurialSettings s = MercurialPlugin::instance()->settings();
-    s.setValue(MercurialSettings::binaryPathKey, m_ui.commandChooser->path());
+    s.setValue(MercurialSettings::binaryPathKey, m_ui.commandChooser->rawPath());
     s.setValue(MercurialSettings::userNameKey, m_ui.defaultUsernameLineEdit->text().trimmed());
     s.setValue(MercurialSettings::userEmailKey, m_ui.defaultEmailLineEdit->text().trimmed());
     s.setValue(MercurialSettings::logCountKey, m_ui.logEntriesCount->value());
@@ -94,16 +91,8 @@ QString OptionsPageWidget::searchKeywords() const
 
 OptionsPage::OptionsPage()
 {
-}
-
-QString OptionsPage::id() const
-{
-    return QLatin1String(VCSBase::Constants::VCS_ID_MERCURIAL);
-}
-
-QString OptionsPage::displayName() const
-{
-    return tr("Mercurial");
+    setId(QLatin1String(VcsBase::Constants::VCS_ID_MERCURIAL));
+    setDisplayName(tr("Mercurial"));
 }
 
 QWidget *OptionsPage::createPage(QWidget *parent)
@@ -125,7 +114,7 @@ void OptionsPage::apply()
     if (newSettings != plugin->settings()) {
         //assume success and emit signal that settings are changed;
         plugin->setSettings(newSettings);
-        newSettings.writeSettings(Core::ICore::instance()->settings());
+        newSettings.writeSettings(Core::ICore::settings());
         emit settingsChanged();
     }
 }
