@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "appoutputpane.h"
 #include "projectexplorerconstants.h"
@@ -52,14 +49,14 @@
 #include <utils/qtcassert.h>
 #include <utils/outputformatter.h>
 
-#include <QtGui/QAction>
-#include <QtGui/QVBoxLayout>
-#include <QtGui/QTabWidget>
-#include <QtGui/QToolButton>
-#include <QtGui/QTabBar>
-#include <QtGui/QMenu>
+#include <QAction>
+#include <QVBoxLayout>
+#include <QTabWidget>
+#include <QToolButton>
+#include <QTabBar>
+#include <QMenu>
 
-#include <QtCore/QDebug>
+#include <QDebug>
 
 enum { debug = 0 };
 
@@ -68,7 +65,7 @@ using namespace ProjectExplorer::Internal;
 
 static QObject *debuggerCore()
 {
-    return ExtensionSystem::PluginManager::instance()->getObjectByName("DebuggerCore");
+    return ExtensionSystem::PluginManager::getObjectByName(QLatin1String("DebuggerCore"));
 }
 
 static QString msgAttachDebuggerTooltip(const QString &handleDescription = QString())
@@ -125,7 +122,7 @@ AppOutputPane::AppOutputPane() :
     m_attachButton(new QToolButton)
 {
     // Rerun
-    m_reRunButton->setIcon(QIcon(ProjectExplorer::Constants::ICON_RUN_SMALL));
+    m_reRunButton->setIcon(QIcon(QLatin1String(ProjectExplorer::Constants::ICON_RUN_SMALL)));
     m_reRunButton->setToolTip(tr("Re-run this run-configuration"));
     m_reRunButton->setAutoRaise(true);
     m_reRunButton->setEnabled(false);
@@ -133,7 +130,6 @@ AppOutputPane::AppOutputPane() :
             this, SLOT(reRunRunControl()));
 
     // Stop
-    Core::ActionManager *am = Core::ICore::instance()->actionManager();
     Core::Context globalcontext(Core::Constants::C_GLOBAL);
 
     QIcon stopIcon = QIcon(QLatin1String(Constants::ICON_STOP));
@@ -142,7 +138,7 @@ AppOutputPane::AppOutputPane() :
     m_stopAction->setToolTip(tr("Stop"));
     m_stopAction->setEnabled(false);
 
-    Core::Command *cmd = am->registerAction(m_stopAction, Constants::STOP, globalcontext);
+    Core::Command *cmd = Core::ActionManager::registerAction(m_stopAction, Constants::STOP, globalcontext);
 
     m_stopButton->setDefaultAction(cmd->action());
     m_stopButton->setAutoRaise(true);
@@ -153,7 +149,7 @@ AppOutputPane::AppOutputPane() :
     // Attach
     m_attachButton->setToolTip(msgAttachDebuggerTooltip());
     m_attachButton->setEnabled(false);
-    m_attachButton->setIcon(QIcon(ProjectExplorer::Constants::ICON_DEBUG_SMALL));
+    m_attachButton->setIcon(QIcon(QLatin1String(ProjectExplorer::Constants::ICON_DEBUG_SMALL)));
     m_attachButton->setAutoRaise(true);
 
     connect(m_attachButton, SIGNAL(clicked()),
@@ -174,7 +170,7 @@ AppOutputPane::AppOutputPane() :
 
     m_mainWidget->setLayout(layout);
 
-    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(aboutToUnloadSession()),
+    connect(ProjectExplorerPlugin::instance()->session(), SIGNAL(aboutToUnloadSession(QString)),
             this, SLOT(aboutToUnloadSession()));
     connect(ProjectExplorerPlugin::instance(), SIGNAL(settingsChanged()),
             this, SLOT(updateFromSettings()));
@@ -378,7 +374,7 @@ void AppOutputPane::showTabFor(RunControl *rc)
 void AppOutputPane::reRunRunControl()
 {
     const int index = currentIndex();
-    QTC_ASSERT(index != -1 && !m_runControlTabs.at(index).runControl->isRunning(), return;)
+    QTC_ASSERT(index != -1 && !m_runControlTabs.at(index).runControl->isRunning(), return);
 
     RunControlTab &tab = m_runControlTabs[index];
 
@@ -399,7 +395,7 @@ void AppOutputPane::attachToRunControl()
 void AppOutputPane::stopRunControl()
 {
     const int index = currentIndex();
-    QTC_ASSERT(index != -1 && m_runControlTabs.at(index).runControl->isRunning(), return;)
+    QTC_ASSERT(index != -1 && m_runControlTabs.at(index).runControl->isRunning(), return);
 
     RunControl *rc = m_runControlTabs.at(index).runControl;
     if (rc->isRunning() && optionallyPromptToStop(rc))
@@ -427,8 +423,8 @@ bool AppOutputPane::closeTab(int index)
 
 bool AppOutputPane::closeTab(int tabIndex, CloseTabMode closeTabMode)
 {
-    const int index = indexOf(m_tabWidget->widget(tabIndex));
-    QTC_ASSERT(index != -1, return true;)
+    int index = indexOf(m_tabWidget->widget(tabIndex));
+    QTC_ASSERT(index != -1, return true);
 
     RunControlTab &tab = m_runControlTabs[index];
 
@@ -441,13 +437,29 @@ bool AppOutputPane::closeTab(int tabIndex, CloseTabMode closeTabMode)
         case CloseTabNoPrompt:
             break;
         case CloseTabWithPrompt:
+            QWidget *tabWidget = m_tabWidget->widget(tabIndex);
             if (!tab.runControl->promptToStop())
                 return false;
+            // The event loop has run, thus the ordering might have changed, a tab might
+            // have been closed, so do some strange things...
+            tabIndex = m_tabWidget->indexOf(tabWidget);
+            index = indexOf(tabWidget);
+            if (tabIndex == -1 || index == -1)
+                return false;
+            tab = m_runControlTabs[index];
             break;
         }
-        if (tab.runControl->stop() == RunControl::AsynchronousStop) {
-            tab.asyncClosing = true;
-            return false;
+        if (tab.runControl->isRunning()) { // yes it might have stopped already, then just close
+            QWidget *tabWidget = m_tabWidget->widget(tabIndex);
+            if (tab.runControl->stop() == RunControl::AsynchronousStop) {
+                tab.asyncClosing = true;
+                return false;
+            }
+            tabIndex = m_tabWidget->indexOf(tabWidget);
+            index = indexOf(tabWidget);
+            if (tabIndex == -1 || index == -1)
+                return false;
+            tab = m_runControlTabs[index];
         }
     }
 
@@ -550,7 +562,7 @@ void AppOutputPane::slotRunControlFinished2(RunControl *sender)
 {
     const int senderIndex = indexOf(sender);
 
-    QTC_ASSERT(senderIndex != -1, return; )
+    QTC_ASSERT(senderIndex != -1, return);
 
     // Enable buttons for current
     RunControl *current = currentRunControl();
