@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "projectmodels.h"
 
@@ -40,11 +37,11 @@
 #include <coreplugin/fileiconprovider.h>
 #include <utils/qtcassert.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QFileInfo>
+#include <QDebug>
+#include <QFileInfo>
 
-#include <QtGui/QFont>
-#include <QtGui/QIcon>
+#include <QFont>
+#include <QIcon>
 
 using namespace ProjectExplorer;
 using namespace ProjectExplorer::Internal;
@@ -54,14 +51,6 @@ namespace {
 
 // sorting helper function
 
-int fileNameCompare(const QString &a, const QString &b)
-{
-    int result = a.compare(b, Qt::CaseInsensitive);
-    if (result != 0)
-        return result;
-    return a.compare(b, Qt::CaseSensitive);
-}
-
 bool sortNodes(Node *n1, Node *n2)
 {
     // Ordering is: project files, project, folder, file
@@ -77,7 +66,7 @@ bool sortNodes(Node *n1, Node *n2)
             const QString fileName1 = QFileInfo(file1->path()).fileName();
             const QString fileName2 = QFileInfo(file2->path()).fileName();
 
-            int result = fileNameCompare(fileName1, fileName2);
+            int result = caseFriendlyCompare(fileName1, fileName2);
             if (result != 0)
                 return result < 0;
             else
@@ -97,7 +86,7 @@ bool sortNodes(Node *n1, Node *n2)
             ProjectNode *project1 = static_cast<ProjectNode*>(n1);
             ProjectNode *project2 = static_cast<ProjectNode*>(n2);
 
-            int result = fileNameCompare(project1->displayName(), project2->displayName());
+            int result = caseFriendlyCompare(project1->displayName(), project2->displayName());
             if (result != 0)
                 return result < 0;
             else
@@ -109,12 +98,35 @@ bool sortNodes(Node *n1, Node *n2)
     if (n2Type == ProjectNodeType)
         return false;
 
+    if (n1Type == VirtualFolderNodeType) {
+        if (n2Type == VirtualFolderNodeType) {
+            VirtualFolderNode *folder1 = static_cast<VirtualFolderNode *>(n1);
+            VirtualFolderNode *folder2 = static_cast<VirtualFolderNode *>(n2);
+
+            if (folder1->priority() > folder2->priority())
+                return true;
+            if (folder1->priority() < folder2->priority())
+                return false;
+            int result = caseFriendlyCompare(folder1->path(), folder2->path());
+            if (result != 0)
+                return result < 0;
+            else
+                return folder1 < folder2;
+        } else {
+            return true; // virtual folder is before folder
+        }
+    }
+
+    if (n2Type == VirtualFolderNodeType)
+        return false;
+
+
     if (n1Type == FolderNodeType) {
         if (n2Type == FolderNodeType) {
             FolderNode *folder1 = static_cast<FolderNode*>(n1);
             FolderNode *folder2 = static_cast<FolderNode*>(n2);
 
-            int result = fileNameCompare(folder1->path(), folder2->path());
+            int result = caseFriendlyCompare(folder1->path(), folder2->path());
             if (result != 0)
                 return result < 0;
             else
@@ -134,11 +146,11 @@ bool sortNodes(Node *n1, Node *n2)
         const QString fileName1 = QFileInfo(filePath1).fileName();
         const QString fileName2 = QFileInfo(filePath2).fileName();
 
-        int result = fileNameCompare(fileName1, fileName2);
+        int result = caseFriendlyCompare(fileName1, fileName2);
         if (result != 0) {
             return result < 0; // sort by filename
         } else {
-            result = fileNameCompare(filePath1, filePath2);
+            result = caseFriendlyCompare(filePath1, filePath2);
             if (result != 0) {
                 return result < 0; // sort by filepath
             } else {
@@ -168,23 +180,23 @@ FlatModel::FlatModel(SessionNode *rootNode, QObject *parent)
     connect(watcher, SIGNAL(hasBuildTargetsChanged(ProjectExplorer::ProjectNode*)),
             this, SLOT(hasBuildTargetsChanged(ProjectExplorer::ProjectNode*)));
 
-    connect(watcher, SIGNAL(foldersAboutToBeAdded(FolderNode *, const QList<FolderNode*> &)),
-            this, SLOT(foldersAboutToBeAdded(FolderNode *, const QList<FolderNode*> &)));
+    connect(watcher, SIGNAL(foldersAboutToBeAdded(FolderNode*,QList<FolderNode*>)),
+            this, SLOT(foldersAboutToBeAdded(FolderNode*,QList<FolderNode*>)));
     connect(watcher, SIGNAL(foldersAdded()),
             this, SLOT(foldersAdded()));
 
-    connect(watcher, SIGNAL(foldersAboutToBeRemoved(FolderNode *, const QList<FolderNode*> &)),
-            this, SLOT(foldersAboutToBeRemoved(FolderNode *, const QList<FolderNode*> &)));
+    connect(watcher, SIGNAL(foldersAboutToBeRemoved(FolderNode*,QList<FolderNode*>)),
+            this, SLOT(foldersAboutToBeRemoved(FolderNode*,QList<FolderNode*>)));
     connect(watcher, SIGNAL(foldersRemoved()),
             this, SLOT(foldersRemoved()));
 
-    connect(watcher, SIGNAL(filesAboutToBeAdded(FolderNode *,const QList<FileNode*> &)),
-            this, SLOT(filesAboutToBeAdded(FolderNode *, const QList<FileNode *> &)));
+    connect(watcher, SIGNAL(filesAboutToBeAdded(FolderNode*,QList<FileNode*>)),
+            this, SLOT(filesAboutToBeAdded(FolderNode*,QList<FileNode*>)));
     connect(watcher, SIGNAL(filesAdded()),
             this, SLOT(filesAdded()));
 
-    connect(watcher, SIGNAL(filesAboutToBeRemoved(FolderNode *, const QList<FileNode*> &)),
-            this, SLOT(filesAboutToBeRemoved(FolderNode *, const QList<FileNode*> &)));
+    connect(watcher, SIGNAL(filesAboutToBeRemoved(FolderNode*,QList<FileNode*>)),
+            this, SLOT(filesAboutToBeRemoved(FolderNode*,QList<FileNode*>)));
     connect(watcher, SIGNAL(filesRemoved()),
             this, SLOT(filesRemoved()));
 }
@@ -248,14 +260,11 @@ QVariant FlatModel::data(const QModelIndex &index, int role) const
         switch (role) {
         case Qt::DisplayRole:
         case Qt::EditRole: {
-            if (folderNode)
-                result = folderNode->displayName();
-            else
-                result = QFileInfo(node->path()).fileName(); //TODO cache that?
+            result = node->displayName();
             break;
         }
         case Qt::ToolTipRole: {
-            result = QDir::toNativeSeparators(node->path());
+            result = node->tooltip();
             break;
         }
         case Qt::DecorationRole: {
@@ -532,7 +541,7 @@ bool FlatModel::filter(Node *node) const
     } else if (ProjectNode *projectNode = qobject_cast<ProjectNode*>(node)) {
         if (m_filterProjects && projectNode->parentFolderNode() != m_rootNode)
             isHidden = !projectNode->hasBuildTargets();
-    } else if (node->nodeType() == FolderNodeType) {
+    } else if (node->nodeType() == FolderNodeType || node->nodeType() == VirtualFolderNodeType) {
         if (m_filterProjects)
             isHidden = true;
     } else if (FileNode *fileNode = qobject_cast<FileNode*>(node)) {
@@ -652,7 +661,6 @@ void FlatModel::removed(FolderNode* parentNode, const QList<Node*> &newNodeList)
     QList<Node *>::const_iterator oldIter = oldNodeList.constBegin();
     QList<Node *>::const_iterator newIter = newNodeList.constBegin();
 
-    Q_ASSERT(isSorted(oldNodeList));
     Q_ASSERT(isSorted(newNodeList));
 
     QSet<Node *> emptyDifference;
@@ -826,3 +834,18 @@ void FlatModel::filesRemoved()
 {
     // Do nothing
 }
+
+namespace ProjectExplorer {
+namespace Internal {
+
+int caseFriendlyCompare(const QString &a, const QString &b)
+{
+    int result = a.compare(b, Qt::CaseInsensitive);
+    if (result != 0)
+        return result;
+    return a.compare(b, Qt::CaseSensitive);
+}
+
+}
+}
+
