@@ -1,34 +1,31 @@
-/**************************************************************************
+/****************************************************************************
 **
-** This file is part of Qt Creator
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
-** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
-**
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** This file is part of Qt Creator.
 **
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU Lesser General Public License version 2.1 requirements
+** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this file.
-** Please review the following information to ensure the GNU Lesser General
-** Public License version 2.1 requirements will be met:
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** Other Usage
-**
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
-**
-**************************************************************************/
+****************************************************************************/
 
 #include "nodemetainfo.h"
 #include "model.h"
@@ -38,9 +35,10 @@
 #include "metainfo.h"
 #include <model.h>
 #include <rewriterview.h>
+#include <propertyparser.h>
 
 #include <QSharedData>
-#include <QtDebug>
+#include <QDebug>
 #include <QIcon>
 #include <QDeclarativeContext>
 #include <QDeclarativeEngine>
@@ -52,8 +50,6 @@
 #include <qmljs/qmljsscopechain.h>
 #include <qmljs/parser/qmljsast_p.h>
 #include <languageutils/fakemetaobject.h>
-#include <private/qdeclarativemetatype_p.h>
-#include <private/qdeclarativestringconverters_p.h>
 
 namespace QmlDesigner {
 
@@ -96,14 +92,14 @@ class PropertyMemberProcessor : public MemberProcessor
 public:
     virtual bool processProperty(const QString &name, const Value *value)
     {
-        const ASTPropertyReference *ref = dynamic_cast<const ASTPropertyReference*>(value);
+        const ASTPropertyReference *ref = value_cast<ASTPropertyReference>(value);
         if (ref) {
             QString type = "unknown";
             if (!ref->ast()->memberType.isEmpty())
                 type = ref->ast()->memberType.toString();
             m_properties.append(qMakePair(name, type));
         } else {
-            if (const QmlObjectValue * ov = dynamic_cast<const QmlObjectValue *>(value)) {
+            if (const CppComponentValue * ov = value_cast<CppComponentValue>(value)) {
                 QString qualifiedTypeName = ov->moduleName().isEmpty() ? ov->className() : ov->moduleName() + '.' + ov->className();
                 m_properties.append(qMakePair(name, qualifiedTypeName));
             } else {
@@ -135,12 +131,12 @@ static inline bool isValueType(const QString &type)
     return objectValuesList.contains(type);
 }
 
-const QmlObjectValue *findQmlPrototype(const ObjectValue *ov, const ContextPtr &context)
+const CppComponentValue *findQmlPrototype(const ObjectValue *ov, const ContextPtr &context)
 {
     if (!ov)
         return 0;
 
-    const QmlObjectValue * qmlValue = dynamic_cast<const QmlObjectValue *>(ov);
+    const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov);
     if (qmlValue)
         return qmlValue;
 
@@ -154,7 +150,7 @@ QStringList prototypes(const ObjectValue *ov, const ContextPtr &context, bool ve
         return list;
     ov = ov->prototype(context);
     while (ov) {
-        const QmlObjectValue * qmlValue = dynamic_cast<const QmlObjectValue *>(ov);
+        const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov);
         if (qmlValue) {
             if (versions) {
                 list << qmlValue->moduleName() + '.' + qmlValue->className() +
@@ -177,11 +173,13 @@ QStringList prototypes(const ObjectValue *ov, const ContextPtr &context, bool ve
 
 QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &context, bool local = false);
 
-QList<PropertyInfo> getQmlTypes(const QmlObjectValue *ov, const ContextPtr &context, bool local = false)
+QList<PropertyInfo> getQmlTypes(const CppComponentValue *ov, const ContextPtr &context, bool local = false)
 {
     QList<PropertyInfo> list;
     if (!ov)
         return list;
+    if (ov->className().isEmpty())
+        return list;
 
     PropertyMemberProcessor processor;
     ov->processMembers(&processor);
@@ -192,7 +190,7 @@ QList<PropertyInfo> getQmlTypes(const QmlObjectValue *ov, const ContextPtr &cont
         QString name = property.first;
         if (!ov->isWritable(name) && ov->isPointer(name)) {
             //dot property
-            const QmlObjectValue * qmlValue = dynamic_cast<const QmlObjectValue *>(ov->lookupMember(name, context));
+            const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov->lookupMember(name, context));
             if (qmlValue) {
                 QList<PropertyInfo> dotProperties = getQmlTypes(qmlValue, context);
                 foreach (const PropertyInfo &propertyInfo, dotProperties) {
@@ -204,7 +202,7 @@ QList<PropertyInfo> getQmlTypes(const QmlObjectValue *ov, const ContextPtr &cont
             }
         }
         if (isValueType(ov->propertyType(name))) {
-            const ObjectValue *dotObjectValue = dynamic_cast<const ObjectValue *>(ov->lookupMember(name, context));
+            const ObjectValue *dotObjectValue = value_cast<ObjectValue>(ov->lookupMember(name, context));
             if (dotObjectValue) {
                 QList<PropertyInfo> dotProperties = getObjectTypes(dotObjectValue, context);
                 foreach (const PropertyInfo &propertyInfo, dotProperties) {
@@ -224,7 +222,7 @@ QList<PropertyInfo> getQmlTypes(const QmlObjectValue *ov, const ContextPtr &cont
     if (!local) {
         const ObjectValue* prototype = ov->prototype(context);
 
-        const QmlObjectValue * qmlObjectValue = dynamic_cast<const QmlObjectValue *>(prototype);
+        const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(prototype);
 
         if (qmlObjectValue) {
             list << getQmlTypes(qmlObjectValue, context);
@@ -240,7 +238,7 @@ QList<PropertyInfo> getTypes(const ObjectValue *ov, const ContextPtr &context, b
 {
     QList<PropertyInfo> list;
 
-    const QmlObjectValue * qmlObjectValue = dynamic_cast<const QmlObjectValue *>(ov);
+    const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(ov);
 
     if (qmlObjectValue) {
         list << getQmlTypes(qmlObjectValue, context, local);
@@ -256,6 +254,9 @@ QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &cont
     QList<PropertyInfo> list;
     if (!ov)
         return list;
+    if (ov->className().isEmpty())
+        return list;
+
     PropertyMemberProcessor processor;
     ov->processMembers(&processor);
 
@@ -264,7 +265,10 @@ QList<PropertyInfo> getObjectTypes(const ObjectValue *ov, const ContextPtr &cont
     if (!local) {
         const ObjectValue* prototype = ov->prototype(context);
 
-        const QmlObjectValue * qmlObjectValue = dynamic_cast<const QmlObjectValue *>(prototype);
+        if (prototype == ov)
+            return list;
+
+        const CppComponentValue * qmlObjectValue = value_cast<CppComponentValue>(prototype);
 
         if (qmlObjectValue) {
             list << getQmlTypes(qmlObjectValue, context);
@@ -358,13 +362,13 @@ public:
 private:
     NodeMetaInfoPrivate(Model *model, QString type, int maj = -1, int min = -1);
 
-    const QmlJS::QmlObjectValue *getQmlObjectValue() const;
+    const QmlJS::CppComponentValue *getCppComponentValue() const;
     const QmlJS::ObjectValue *getObjectValue() const;
     void setupPropertyInfo(QList<PropertyInfo> propertyInfos);
     void setupLocalPropertyInfo(QList<PropertyInfo> propertyInfos);
     QString lookupName() const;
     QStringList lookupNameComponent() const;
-    const QmlJS::QmlObjectValue *getNearestQmlObjectValue() const;
+    const QmlJS::CppComponentValue *getNearestCppComponentValue() const;
     QString fullQualifiedImportAliasType() const;
 
     QString m_qualfiedTypeName;
@@ -382,7 +386,7 @@ private:
 
     //storing the pointer would not be save
     QmlJS::ContextPtr context() const;
-    QmlJS::Document *document() const;
+    const Document *document() const;
 
     QPointer<Model> m_model;
     static QHash<QString, Pointer> m_nodeMetaInfoCache;
@@ -424,7 +428,7 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, QString type, int maj, in
                                         m_model(model)
 {
     if (context()) {
-        const QmlObjectValue *objectValue = getQmlObjectValue();
+        const CppComponentValue *objectValue = getCppComponentValue();
         if (objectValue) {
             setupPropertyInfo(getTypes(objectValue, context()));
             setupLocalPropertyInfo(getTypes(objectValue, context(), true));
@@ -434,7 +438,7 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, QString type, int maj, in
         } else {
             const ObjectValue *objectValue = getObjectValue();
             if (objectValue) {
-                const QmlObjectValue *qmlValue = dynamic_cast<const QmlObjectValue *>(objectValue);
+                const CppComponentValue *qmlValue = value_cast<CppComponentValue>(objectValue);
                 if (qmlValue) {
                     m_majorVersion = qmlValue->componentVersion().majorVersion();
                     m_minorVersion = qmlValue->componentVersion().minorVersion();
@@ -452,7 +456,7 @@ NodeMetaInfoPrivate::NodeMetaInfoPrivate(Model *model, QString type, int maj, in
     }
 }
 
-const QmlJS::QmlObjectValue *NodeMetaInfoPrivate::getQmlObjectValue() const
+const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getCppComponentValue() const
 {
     const QStringList nameComponents = m_qualfiedTypeName.split('.');
     if (nameComponents.size() < 2)
@@ -460,7 +464,7 @@ const QmlJS::QmlObjectValue *NodeMetaInfoPrivate::getQmlObjectValue() const
     const QString type = nameComponents.last();
 
     // maybe 'type' is a cpp name
-    const QmlJS::QmlObjectValue *value = context()->valueOwner()->cppQmlTypes().objectByCppName(type);
+    const QmlJS::CppComponentValue *value = context()->valueOwner()->cppQmlTypes().objectByCppName(type);
     if (value)
         return value;
 
@@ -476,7 +480,7 @@ const QmlJS::QmlObjectValue *NodeMetaInfoPrivate::getQmlObjectValue() const
         if (import.info.path() != module)
             continue;
         const Value *lookupResult = import.object->lookupMember(type, context());
-        if ((value = dynamic_cast<const QmlObjectValue *>(lookupResult)))
+        if ((value = value_cast<CppComponentValue>(lookupResult)))
             return value;
     }
 
@@ -490,13 +494,13 @@ const QmlJS::ObjectValue *NodeMetaInfoPrivate::getObjectValue() const
 
 QmlJS::ContextPtr NodeMetaInfoPrivate::context() const
 {
-    if (m_model && m_model->rewriterView()) {
-        return m_model->rewriterView()->scopeChain().context();
+    if (m_model && m_model->rewriterView() && m_model->rewriterView()->scopeChain()) {
+        return m_model->rewriterView()->scopeChain()->context();
     }
     return QmlJS::ContextPtr(0);
 }
 
-QmlJS::Document *NodeMetaInfoPrivate::document() const
+const QmlJS::Document *NodeMetaInfoPrivate::document() const
 {
     if (m_model && m_model->rewriterView()) {
         return m_model->rewriterView()->document();
@@ -541,7 +545,7 @@ bool NodeMetaInfoPrivate::isPropertyWritable(const QString &propertyName) const
             return true;
     }
 
-    const QmlJS::QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return true;
     if (qmlObjectValue->hasProperty(propertyName))
@@ -573,7 +577,7 @@ bool NodeMetaInfoPrivate::isPropertyList(const QString &propertyName) const
             return true;
     }
 
-    const QmlJS::QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
     return qmlObjectValue->isListProperty(propertyName);
@@ -601,7 +605,7 @@ bool NodeMetaInfoPrivate::isPropertyPointer(const QString &propertyName) const
             return true;
     }
 
-    const QmlJS::QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const QmlJS::CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
     return qmlObjectValue->isPointer(propertyName);
@@ -629,7 +633,7 @@ bool NodeMetaInfoPrivate::isPropertyEnum(const QString &propertyName) const
             return false;
     }
 
-    const QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return false;
     return qmlObjectValue->getEnum(propertyType(propertyName)).isValid();
@@ -657,10 +661,10 @@ QString NodeMetaInfoPrivate::propertyEnumScope(const QString &propertyName) cons
             return QString();
     }
 
-    const QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return QString();
-    const QmlObjectValue *definedIn = 0;
+    const CppComponentValue *definedIn = 0;
     qmlObjectValue->getEnum(propertyType(propertyName), &definedIn);
     if (definedIn)
         return definedIn->className();
@@ -686,7 +690,7 @@ bool NodeMetaInfoPrivate::cleverCheckType(const QString &otherType) const
     if (packageName() == package)
         return QString(package + '.' + typeName) == qualfiedTypeName();
 
-    const QmlObjectValue *qmlObjectValue = getQmlObjectValue();
+    const CppComponentValue *qmlObjectValue = getCppComponentValue();
     if (!qmlObjectValue)
         return false;
 
@@ -741,7 +745,7 @@ QStringList NodeMetaInfoPrivate::keysForEnum(const QString &enumName) const
     if (!isValid())
         return QStringList();
 
-    const QmlObjectValue *qmlObjectValue = getNearestQmlObjectValue();
+    const CppComponentValue *qmlObjectValue = getNearestCppComponentValue();
     if (!qmlObjectValue)
         return QStringList();
     return qmlObjectValue->getEnum(enumName).keys();
@@ -750,7 +754,7 @@ QStringList NodeMetaInfoPrivate::keysForEnum(const QString &enumName) const
 QString NodeMetaInfoPrivate::packageName() const
 {
     if (!isComponent()) {
-        if (const QmlObjectValue *qmlObject = getQmlObjectValue())
+        if (const CppComponentValue *qmlObject = getCppComponentValue())
             return qmlObject->moduleName();
     }
     return QString();
@@ -759,7 +763,7 @@ QString NodeMetaInfoPrivate::packageName() const
 QString NodeMetaInfoPrivate::componentSource() const
 {
     if (isComponent()) {
-        const ASTObjectValue * astObjectValue = dynamic_cast<const ASTObjectValue *>(getObjectValue());
+        const ASTObjectValue * astObjectValue = value_cast<ASTObjectValue>(getObjectValue());
         if (astObjectValue)
             return astObjectValue->document()->source().mid(astObjectValue->typeName()->identifierToken.begin(),
                                                             astObjectValue->initializer()->rbraceToken.end());
@@ -770,7 +774,7 @@ QString NodeMetaInfoPrivate::componentSource() const
 QString NodeMetaInfoPrivate::componentFileName() const
 {
     if (isComponent()) {
-        const ASTObjectValue * astObjectValue = dynamic_cast<const ASTObjectValue *>(getObjectValue());
+        const ASTObjectValue * astObjectValue = value_cast<ASTObjectValue>(getObjectValue());
         if (astObjectValue) {
             QString fileName;
             int line;
@@ -825,14 +829,14 @@ void NodeMetaInfoPrivate::setupPrototypes()
     if (m_isComponent)
         objects = PrototypeIterator(getObjectValue(), context()).all();
     else
-        objects = PrototypeIterator(getQmlObjectValue(), context()).all();
+        objects = PrototypeIterator(getCppComponentValue(), context()).all();
 
     foreach (const ObjectValue *ov, objects) {
         TypeDescription description;
         description.className = ov->className();
         description.minorVersion = -1;
         description.majorVersion = -1;
-        if (const QmlObjectValue * qmlValue = dynamic_cast<const QmlObjectValue *>(ov)) {
+        if (const CppComponentValue * qmlValue = value_cast<CppComponentValue>(ov)) {
             description.minorVersion = qmlValue->componentVersion().minorVersion();
             description.majorVersion = qmlValue->componentVersion().majorVersion();
             LanguageUtils::FakeMetaObject::Export qtquickExport = qmlValue->metaObject()->exportInPackage("QtQuick");
@@ -858,11 +862,11 @@ QList<TypeDescription> NodeMetaInfoPrivate::prototypes() const
     return m_prototypes;
 }
 
-const QmlJS::QmlObjectValue *NodeMetaInfoPrivate::getNearestQmlObjectValue() const
+const QmlJS::CppComponentValue *NodeMetaInfoPrivate::getNearestCppComponentValue() const
 {
     if (m_isComponent)
         return findQmlPrototype(getObjectValue(), context());
-    return getQmlObjectValue();
+    return getCppComponentValue();
 }
 
 QString NodeMetaInfoPrivate::fullQualifiedImportAliasType() const
@@ -998,8 +1002,7 @@ QVariant NodeMetaInfo::propertyCastedValue(const QString &propertyName, const QV
         return variant;
     }
 
-    return QDeclarativeStringConverters::variantFromString(variant.toString());
-
+    return Internal::PropertyParser::variantFromString(variant.toString());
 }
 
 QList<NodeMetaInfo> NodeMetaInfo::superClasses() const
